#include "clipboardfiles.h"

bool ClipboardFiles::isEmpty()
{
	return ( entries().first.count() == 0 );
}

QPair<QList<QFileInfo>,bool> ClipboardFiles::entries()
{
	const QClipboard *clipboard = QGuiApplication::clipboard();
	const QMimeData *mimeData = clipboard->mimeData(QClipboard::Clipboard);
	return entries( mimeData );
}

QPair<QList<QFileInfo>,bool> ClipboardFiles::entries( const QMimeData *mimeData )
{ 
	QPair<QList<QFileInfo>,bool> ret;
	QStringList formattedMimes = QString( FILE_MIMETYPE_FORMAT_LIST ).split(',');
	bool parsed = false;
	for ( int i = 0; ! parsed && i < formattedMimes.count(); i++ )
	{
		if ( ! mimeData->data( formattedMimes[i] ).isEmpty() )
		{
			QList<QByteArray> entries = mimeData->data( formattedMimes[i] ).split('\0');
			for ( int j = 0; j < entries.count(); j++ )
			{
				if ( QString(entries[j]).startsWith("cut", Qt::CaseInsensitive) )
				{
					ret.second = true;
				} else if ( QString(entries[j]).startsWith("copy", Qt::CaseInsensitive) ) {
					ret.second = false;
				} else {
					continue;
				}
				QString entry = QString(entries[j]).section('\n',1,1);
				entry.remove("file://");
				ret.first += QFileInfo( entry );
				parsed = true;
			}
		}
	}
	if ( ! parsed )
	{
		QString data = QUrl::fromPercentEncoding( mimeData->data( "text/uri-list" ) );
		QStringList entries = data.split("\n");

		for ( int i = 0; i < entries.count(); i++)
		{
			QString entry = entries[i];
#ifdef Q_OS_WIN
			if ( entry.startsWith("file:///") ) // Local unit (file:///C:/tmp)
			{
				entry.remove("file:///");
				ret.first += QFileInfo(entry);
			} else  if ( entry.startsWith("file:") ) { // Network share (file://server/sharedir)
				entry.remove("file:");
				ret.first += QFileInfo(entry);
			}
#else
			if ( entry.startsWith("file://") )
			{

				entry.remove("file://");
				ret.first += QFileInfo(entry);
			}
#endif
		}
	}
	return ret;
}

void ClipboardFiles::setEntries(QList<QFileInfo> entries, bool cut)
{
	QClipboard *clipboard = QGuiApplication::clipboard();
	QMimeData * mimeData = new QMimeData;
	QByteArray operation;
	if ( cut )
	{
		operation = "cut";
	} else {
		operation = "copy";
	}

	for ( int i = 0; i < entries.count(); i++ )
	{
		if ( ! entries[i].absoluteFilePath().isEmpty() )
		{
			QByteArray data = "file://" + entries[i].absoluteFilePath().toUtf8();
			mimeData->setData( "text/uri-list", data );
#if defined(Q_OS_LINUX) || defined(Q_OS_FREEBSD)
			data = operation + "\n" + data + "\0";
			QStringList formattedMimes = QString( FILE_MIMETYPE_FORMAT_LIST ).split(',');
			for ( int i = 0; i < formattedMimes.count(); i++ )
			{
				mimeData->setData( formattedMimes[i], data );
			}
#endif
			clipboard->setMimeData( mimeData, QClipboard::Clipboard);
		}
	}
	/* mimeData must not be deleted, it will be deleted when needed by QClipboard */
}
