#include "cloudconfig.h"

CloudConfig::CloudConfig(QWidget *parent ) : QDialog( parent )
{
	setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	init ();
	loadSettings();
	selectionChanged();
}

CloudConfig::~CloudConfig()
{
	if ( LWClient::instance()->isConnected() )
	{
		qDebug("Umounting cloud drives...");
		QString ret = LWClient::instance()->sendMessage("umount_cloud_drives");
		qDebug("Status: %s", ret.toUtf8().data());
	}
}

void CloudConfig::init()
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	QFont bFont;
	bFont.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );

	QLabel * cloudListLabel = new QLabel(tr("Cloud drives:"), this);
	cloudListLabel->setMinimumWidth( 100 );
	cloudListLabel->setFont( bFont );
	cloudList = new QListWidget( this );
	cloudList->setMinimumWidth( 250 );
	cloudList->setSelectionMode( QAbstractItemView::SingleSelection );
	connect(cloudList, SIGNAL(itemSelectionChanged()), this, SLOT(selectionChanged()));

	appendButton = new QPushButton( QIcon(":/icons/32x32/add.png"), "", this);
	appendButton->setFlat( true );
	appendButton->setFixedSize( TOOLBUTTON_SIZE + ICONMARGIN, TOOLBUTTON_SIZE + ICONMARGIN );
	appendButton->setIconSize( QSize(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE) );
	removeButton = new QPushButton( QIcon(":/icons/32x32/remove.png"), "", this);
	removeButton->setFlat( true );
	removeButton->setFixedSize( TOOLBUTTON_SIZE + ICONMARGIN, TOOLBUTTON_SIZE + ICONMARGIN );
	removeButton->setIconSize( QSize(TOOLBUTTON_SIZE, TOOLBUTTON_SIZE) );

	layout->addWidget(cloudListLabel, 1, 0);
	layout->addWidget(cloudList, 1, 1, 4, 5);
	layout->addWidget(appendButton, 1, 6);
	layout->addWidget(removeButton, 2, 6);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect(appendButton, SIGNAL(clicked()), this, SLOT(append()));
	connect(removeButton, SIGNAL(clicked()), this, SLOT(remove()));
}

void CloudConfig::loadSettings()
{
	if ( LWClient::instance()->isConnected() )
	{
		QString token;
		qDebug("Looking for token on file: %s", QString(TOKEN_FILE).toUtf8().data());
		if (QFile::exists( TOKEN_FILE ))
		{
			token = Settings::getValue( TOKEN_FILE, "UserToken" );
		} else {
			return;
		}

		qDebug("Mounting cloud drives...");
		QString ret = LWClient::instance()->sendMessage("mount_cloud_drives");
		qDebug("Status: %s", ret.toUtf8().data());

		cloudList->clear();
		cloudListTypes.clear();
		QStringList cloudTypes;
		//cloudTypes << "dropbox" << "googledrive";
		cloudTypes << "googledrive";
		QStringList cloudTypeNames;
		//cloudTypeNames << "Dropbox" << "Google Drive";
		cloudTypeNames << "Google Drive";
		for ( int i = 0; i < cloudTypes.count(); i++ )
		{
			QString configs = LWClient::instance()->sendMessage("get_cloud_drives?" + cloudTypes[i]).simplified();
			QStringList configList;
			if ( ! configs.isEmpty() )
			{
				configList = configs.split(",");
			}
			for ( int j = 0; j < configList.count(); j++ )
			{
				cloudList->addItem(new QListWidgetItem(QIcon(":/icons/applications/" + cloudTypes[i] + ".png"), configList[j]));
				cloudListTypes.append(cloudTypes[i]);
			}
		}
	}
}

void CloudConfig::append()
{
	CloudConfigWizard * cloudConfigWizard = new CloudConfigWizard(itemStringList());
	QScreen * screen = WinInfo::widgetScreen( this );
	
	int width = 800;
	int height = 600;

	if ( screen != nullptr )
	{
		width = screen->availableSize().width();
		height = screen->availableSize().height();
		if ( width > 800 )
		{
			width = 800;
		}

		if ( height > 600 )
		{
			height = 600;
		}
	}

	cloudConfigWizard->setFixedSize(width, height);
	cloudConfigWizard->setModal( true );
	cloudConfigWizard->setWindowModality(Qt::ApplicationModal);
	cloudConfigWizard->show();
	connect( cloudConfigWizard, SIGNAL(accepted()), this, SLOT(loadSettings()));
	connect( cloudConfigWizard, SIGNAL(rejected()), cloudConfigWizard, SLOT(deleteLater()));
	connect( cloudConfigWizard, SIGNAL(accepted()), cloudConfigWizard, SLOT(deleteLater()));
}

void CloudConfig::remove()
{
	if (cloudList->currentItem() != nullptr)
	{
		QMessageBox::StandardButton button = QMessageBox::warning( this,
					tr("Remove cloud drive"),
					tr("You will loose access to the selected cloud drive, identified by '%1'. Are you sure you want to do this?").arg(cloudList->currentItem()->text()),
					QMessageBox::Yes|QMessageBox::No,
					QMessageBox::No);
		if ( button == QMessageBox::Yes )
		{
			if ( LWClient::instance()->isConnected() )
			{
				QString ret = LWClient::instance()->sendMessage("remove_cloud_drive?" + cloudListTypes[cloudList->currentRow()] + "&" + cloudList->currentItem()->text());
				if ( ret.startsWith("Error:") )
				{
					qDebug() << ret;
				}
				loadSettings();
			}
		}
	}
}

void CloudConfig::selectionChanged()
{
	if ( cloudList->selectedItems().count() > 0 )
	{
		removeButton->setEnabled( true );
	} else {
		removeButton->setEnabled( false );
	}
}

void CloudConfig::accept()
{
	setFocus();
}

void CloudConfig::reject()
{
	setFocus();
}

QStringList CloudConfig::itemStringList()
{
	QStringList ret;
	for ( int i = 0; i < cloudList->count(); i++ )
	{
		ret += cloudList->item(i)->text();
	}
	return ret;
}
