#include "settings.h"

#ifdef Q_OS_LINUX
#define CONF_DIR "/etc/librework-desktop"
#define CONFIG_FILE CONF_DIR "/librework-desktop.ini"
#define VARDIR "/var/librework"
#define MIME_CACHE_DIR Settings::homePath() + "/.local/share/applications"
#define MIME_CACHE_FILE MIME_CACHE_DIR + "/mimeinfo.cache"
#define REMOTE_MIME_APPS_LIST CONF_DIR "remotemimeapps.list"

#ifdef Q_OS_FREEBSD
#define BASEDIR "/usr/local"
#else
#define BASEDIR "/usr"
#endif
#define SHAREDIR BASEDIR "/share/librework-desktop/"
#define CHROOT "schroot"
#define CHROOTMOUNT Settings::getValue("ChrootMount") + "/mnt." + Settings::getUserName()
#define XRDP_DIS BASEDIR "/bin/xrdp-dis"
#define CLIENT_CONFIG_FILE "/etc/librework-client/librework-client.ini"
#define RDP_CLIENT "xfreerdp"
//#define RDP_CLIENT "rdesktop"
#endif

#ifdef Q_OS_WIN
#define PROGRAMFILES qgetenv("PROGRAMW6432")
#define CONF_DIR PROGRAMFILES + "\\Librework Desktop\\"
#define CONFIG_FILE CONF_DIR + "librework-desktop.ini"
#define SHAREDIR PROGRAMFILES + "\\Librework Desktop\\"
#define VARDIR SHAREDIR
#define CLIENT_CONFIG_FILE PROGRAMFILES + "\\Librework Client\\Librework-client.ini"
//#define RDP_CLIENT "wfreerdp.exe"
#define RDP_CLIENT "mstsc.exe"
#endif

#define LWVERSION 2.3

#define WINMANAGER_TIMER 300
#define SYSTRAY_TIMER 500
#define NETWORK_START_WAIT_TIME 10000
#define FILE_SEND_BUFFER_SIZE 65536

#define BASICFSMODE

#define USER_CONFIG Settings::homePath() + QDir::separator() + ".librework.ini"
#define TOKEN_FILE USER_CONFIG

#ifdef Q_OS_LINUX
#ifndef HWND
#define HWND long long
#endif
#ifndef DWORD
#define DWORD long
#endif
#endif

// OS
#define LINUX 1
#define WINDOWS 2

#ifdef Q_OS_LINUX
#define RUNNING_OS LINUX
#endif
#ifdef Q_OS_WIN
#define RUNNING_OS WINDOWS
#endif

// Other
#define FILE_MIMETYPE_FORMAT_LIST "application/x-kde-cutselection,x-special/mate-copied-files,x-special/gnome-copied-files"

// Data dirs
#define ICONDIR ":/icons/"
#define ICON_DIR_16 ICONDIR "16x16"
#define ICON_DIR_24 ICONDIR "24x24"
#define ICON_DIR_32 ICONDIR "32x32"
#define ICON_DIR_48 ICONDIR "48x48"
#define ICON_DIR_64 ICONDIR "64x64"
#define ICON_DIR_128 ICONDIR "128x128"
#define ICON_DIR_OTHER ICONDIR "other"
#define ICON_DIR_APPS ICONDIR "applications"
#define ICON_DIR_GROUPS ICONDIR "groups"
#define ICON_DIR_ANIM ICONDIR "animations"
#define ICON_DIR_WM ICONDIR "wm/"
#define WALLPAPERS_DIR SHAREDIR "wallpapers"
#define MIME_DIR ":/mime"
#define I18N_DIR SHAREDIR "i18n"
#define APPLICATIONS_FILE MIME_DIR + QString(QDir::separator()) + "applications"
#define GROUPS_FILE MIME_DIR + QString(QDir::separator()) + "groups"
#define EXTENSIONS_FILE MIME_DIR + QString(QDir::separator()) + "extensions"
#define MIMETYPES_FILE MIME_DIR + QString(QDir::separator()) + "types"
#define LIBREWORK_DESKTOP_SERVER_PORT 4445

#define DEFAULT_WALLPAPER "wallpaper-01.png"

#define GOOGLEDRIVE_HOME "Google Drive"
#define DROPBOX_HOME "Dropbox"
#define GOOGLEPRINT_CONF ".cloudprint.conf"
#define GOOGLEPRINT_MANAGER "/usr/lib/cloudprint-cups/manageaccounts.py"
#define SEARCH_DIR "search"
#define DOWNLOADS_DIR "Downloads"
#define SHARED_DIR "/shared"
#define MEDIA_DIR "/media"

#define TIMER_RIGHT_CLICK 1250
#define TIMEOUT 8 // in seconds
#define LOADINGTIMEOUT 20 // in seconds

#define MAXLINKDEPTH 10

#define MAXPREVIEWSIZE 10485760

// Window Manager Behaviour
#define FULLSCREEN 0
#define STANDARD 1

// ColorsbuttonStyleSheet
#define BORDERCOLOR "#a6aabd"
#define CONFIGURATIONBACKGROUNDCOLOR "white"
#define CONFIGURATIONFONTCOLOR "#111111"
#define BACKGROUNDCOLOR "#0f3e72"
#define FONTCOLOR "#e6e6e6"
#define TABCOLOR "#222222"
#define TABSELECTIONCOLOR "#4f4f4f"
#define MENUBACKGROUNDCOLOR "#f2f2f2"
#define MENUBORDERCOLOR "#a0a0a0"
#define MENUHIGHLIGHTCOLOR "#d9d9d9"
#define MENUFONTCOLOR "#000000"
#define MENUDISABLEDCOLOR "#aaaaaa"
#define FILESYSTEMWIDGETFONTCOLOR "#4f4f4f"
#define COMBOSTYLE					"QComboBox {" \
							"	border: 1px solid" BORDERCOLOR ";" \
							"	background: " CONFIGURATIONBACKGROUNDCOLOR ";" \
							"	color: " CONFIGURATIONFONTCOLOR ";" \
							"}"
#define STANDARDMENUSTYLE				"QMenu {" \
							"	border: 1px solid" MENUBORDERCOLOR ";" \
							"	background: " MENUBACKGROUNDCOLOR ";" \
							"	color: " MENUFONTCOLOR ";" \
							"}" \
							"QMenu::item:selected {" \
							"	background: " MENUHIGHLIGHTCOLOR ";" \
							"	color: " MENUFONTCOLOR ";" \
							"}" \
							"QMenu::item:disabled {" \
							"	color: " MENUDISABLEDCOLOR ";" \
							"}"
#define STANDARDBUTTONSTYLE				"QPushButton {" \
							"	border: 1px solid " BORDERCOLOR ";" \
							"	background: #e1e1e1;" \
							"}" \
							"QPushButton:hover {" \
							"	border: 1px solid #0078d7;" \
							"	background: #e5f1fb;" \
							"}" \
							"QPushButton:pressed {" \
							"	border: 1px solid #005499;" \
							"	background: #cce4f7;" \
							"}" \
							"QPushButton:focus {" \
							"	border: 2px solid #0078d7;" \
							"	background: #e1e1e1" \
							"}"

#define STANDARDDIALOGCOLORS				"{" \
							"       background-color: white;" \
							"       color: " FILESYSTEMWIDGETFONTCOLOR ";" \
							"}"
#define STANDARDDIALOGSTYLE	"QDialog" STANDARDDIALOGCOLORS \
							"QMessageBox" STANDARDDIALOGCOLORS \
							"QWidget" STANDARDDIALOGCOLORS \
							"QPushButton" STANDARDDIALOGCOLORS \
							"QDialogButtonBox" STANDARDDIALOGCOLORS \
							"QFrame" STANDARDDIALOGCOLORS \
							"QIcon" STANDARDDIALOGCOLORS \
							"QLabel" STANDARDDIALOGCOLORS

// Icons and buttons sizes
#define MAX_ICON_BYTESIZE 1048576
#define SYSTRAY_ICON_SIZE 24
#define CONFIGURATION_ICON_SIZE 32
#define CLOSEBUTTON_SIZE 16
#define TOOLBUTTON_SIZE 24
#define PLACES_ICON_SIZE 32
#define ICON_SIZE 48
#define LOGO_MENU_ICON_SIZE 32
#define MENU_ICON_SIZE 24

#define TAB_BUTTON_HEIGHT 42
#define TAB_BUTTON_WIDTH 48
#define THREAD_WIDGET_WIDTH 250
#define BUTTON_PRESSED_MARGIN 2

#define SEPARATION 12
#define ICONMARGIN 6
#define SEPARATOR_WIDTH 3 // Width in pixels for desktop separator
#define SEPARATOR_ALPHA 4 // Alpha channel for separator (0-255) (0=transparent,255=opaque)
#define SEPARATOR_MARGIN 20 // Desktop separator margin from top-bottom-right-left

#define DEFAULT_MARGIN 20
#define DEFAULT_SPACING 10

// Fonts
#define LIBREWORK_FONT_DIR SHAREDIR "fonts"
#define LIBREWORK_FONT "WeblySleek UI Normal"
#define LIBREWORK_FONT_BOLD "WeblySleek UI Semibold"
#define LIBREWORK_FONT_BASE_SIZE 12 // Default Librework font pixel size -> Settings::getUserValue("FontSize", QString::number(LIBREWORK_FONT_BASE_SIZE)).toInt()
#define LISTVIEWITEM_FONT_SIZE 10 // Default AppsWidget font pixel size -> Settings::getUserValue("AppButtonFontSize", QString::number(APPBUTTON_FONT_SIZE)).toInt()

// Item for ListView (PlacesWidget, AppsWidget, FileSystemWidget in icon mode)
#define LISTVIEWITEM_ICON_SIZE 48
#define LISTVIEWITEM_TOP_MARGIN DEFAULT_MARGIN
#define LISTVIEWITEM_SEPARATION 6
#define LISTVIEWITEM_TEXT_LINES 2
#define LISTVIEWITEM_FONT_MARGIN 2
#define LISTVIEWITEM_FONT_SEPARATION 2
#define LISTVIEWITEM_LABEL_HEIGHT LISTVIEWITEM_TEXT_LINES * (LISTVIEWITEM_FONT_MARGIN + LISTVIEWITEM_FONT_SIZE) + ( LISTVIEWITEM_TEXT_LINES - 1 ) * LISTVIEWITEM_FONT_SEPARATION
#define LISTVIEWITEM_HEIGHT SEPARATION + LISTVIEWITEM_ICON_SIZE + SEPARATION + LISTVIEWITEM_LABEL_HEIGHT + 2 * SEPARATION
#define LISTVIEWITEM_WIDTH ICON_SIZE + 4 * Settings::getUserValue("AppButtonFontSize", QString::number(LISTVIEWITEM_FONT_SIZE)).toInt()
#define LISTVIEWITEM_GRID_SIZE QSize(LISTVIEWITEM_WIDTH, LISTVIEWITEM_HEIGHT)

#define INITPROGRESS_WIDTH 480
#define INITPROGRESS_HEIGHT 360
#define LOGINSCREEN_OBSCURED_OPACITY 0.7

// ThreadButton types
#define LOGOMENUBUTTON -1
#define MENUBUTTON 0
#define DESKTOPBUTTON 1
#define CURRENTTHREADBUTTON 2
#define THREADBUTTON 3
#define EXITBUTTON 4

// FileManager
#define DBUS_TIMEOUT 5
#define PLACE_HOME 0
#define PLACE_SHARED 1
#define PLACE_SEARCHS 2

// ToolButton
#define TOOLBUTTON_MARGIN 5

