#include "fileproperties.h"
#include "ui_fileproperties.h"

#include <QDir>

FileProperties::FileProperties(QFileInfo file, QWidget *parent) :
	QDialog(parent),
	ui(new Ui::FileProperties)
{
	ui->setupUi(this);
	this->file = QFileInfo(file);
	setWindowTitle( file.baseName() );
	setWindowIcon( QIcon(":/icons/64x64/properties.png") );
	init();
}

FileProperties::~FileProperties()
{
	if ( calculate != 0 )
	{
		calculate->finishThread();
		delete calculate;
	}
	delete ui;
}

void FileProperties::init()
{
	int files = 0;
	int dirs = 0;

	if ( file.isDir() )
	{
		ui->fileIcon->setIcon(QIcon(":/icons/48x48/folder.png"));
		calculate = new FileThread( FileThread::CALCULATE_SIZES, 0, file );
		connect( calculate, SIGNAL(finished()), this, SLOT(setValues()));
		calculate->start();
	} else {
		calculate = 0;
		ui->fileIcon->setIcon( Application::getIcon( file ));
		ui->folders->setEnabled(false);
		ui->foldersLabel->setEnabled(false);
		ui->files->setEnabled(false);
		ui->filesLabel->setEnabled(false);
		files++;
	}
	int extensionPos = file.fileName().lastIndexOf(".");
	if ( extensionPos >= 0 )
	{
		QString baseName = file.fileName();
		baseName.truncate(extensionPos);
		QString extension = file.fileName().right( file.fileName().size() - extensionPos );
		ui->fileName->setText( baseName );
		ui->fileType->setText( extension );
	} else {
		ui->fileName->setText( file.completeBaseName() );
		ui->fileType->setText( tr("Folder") );
	}
	ui->location->setText( file.absoluteDir().absolutePath() );
	if ( file.isDir() )
	{
		ui->size->setText( tr("Calculating folder size...") );
		ui->files->setText( tr("Calculating number of files...") );
		ui->folders->setText( tr("Calculating number of folders...") );
	} else {
		setSize( file.size() );
		ui->folders->setText( QString::number( dirs ) );
		ui->files->setText( QString::number( files ) );
	}
	ui->modification->setText( file.lastModified().toString() );
	ui->accessed->setText( file.lastRead().toString() );
}

void FileProperties::setValues()
{
	if ( calculate != 0 )
	{
		//qDebug("Sizes: %d", calculate->sizes());
		setSize( calculate->sizes() );
		setFileCount( calculate->files() );
		setDirCount( calculate->dirs() );
		delete calculate;
		calculate = 0;
	}
}

void FileProperties::setSize(long long size)
{
	QString human = toHuman( size );
	if ( human.endsWith("Bytes") )
	{
		ui->size->setText( human );
	} else {
		ui->size->setText( QString::number( size ) + " bytes (" + toHuman( size ) + ")" );
	}
}

void FileProperties::setFileCount(int files)
{
	ui->files->setText( QString::number( files ) );
}

void FileProperties::setDirCount(int dirs)
{
	ui->folders->setText( QString::number( dirs ) );
}

QString FileProperties::toHuman(long long size)
{
	QString ret;
	QStringList suffix;
	suffix << "bytes" << "KiB" << "MiB" << "GiB" << "TiB" << "PiB";
	long long rest = size;
	long long multiplier = 1024;
	long long decimal = 0;

	int i = 0;
	while ( rest > 1024 && i < ( suffix.count() - 1 ) )
	{
		long long newRest = rest / multiplier;
		decimal = rest - newRest * multiplier;
		decimal = decimal / ( multiplier / 10 );
		rest = newRest;
		i++;
	}

	if ( decimal == 0 )
	{
		return QString::number( rest ) + " " + suffix[i];
	} else {
		return QString::number( rest ) + "." + QString::number( decimal ) + " " + suffix[i];
	}
}
