#ifndef FILESYSTEMWIDGET_H
#define FILESYSTEMWIDGET_H

#include <QWidget>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QStyleOptionViewItem>
#include <QTimer>
#include <QMenu>
#include <QMouseEvent>
#include <QMessageBox>
#include <QInputDialog>
#include <QDir>
#include <QFile>
#include <QMutex>
#include <QProgressDialog>
#include <QPalette>
#include <QAbstractItemView>
#include <QUuid>

#ifdef Q_OS_WIN
#include <WinSock2.h>
#include <windows.h>
#include <shellapi.h>
#endif

#include "constants.h"
#include "clipboardfiles.h"
#include "filesystemmodel.h"
#include "remoteclientmanager.h"
#include "application.h"
#include "lwclient.h"
#include "fileproperties.h"
#include "placesmodel.h"
#include "listview.h"
#include "iconview.h"

namespace Ui {
class FileSystemWidget;
}

class FileSystemWidget : public QWidget
{
    Q_OBJECT

public:
    explicit FileSystemWidget(QWidget *parent = 0, bool standalone = false, QString openLocation = QString());
	explicit FileSystemWidget(QIcon icon, QWidget *parent = 0); // For Configure class
    ~FileSystemWidget();

    void initUi(QString openLocation);
    static bool copyRecursively(QString sourceFile, QString targetFile);

signals:
	void mousePressed();

public slots:
	void close();
	void dropFiles(QModelIndex target, QStringList fileList, QDropEvent *e);

private slots:
	void setMenu();
	void setNoSelectionMenu();
	void directoryLoaded(QString dir);
	void menuRequested(const QPoint pos);
	void setIndex( QModelIndex index );
	void upButtonPressed();
	void doubleClicked(QModelIndex index);
	void mousePressed(QModelIndex index);
	void fillPlaces();
	void updatePlaces();
	void setPlaceIndex( QModelIndex index );
	void placesDoubleClicked(QModelIndex index);
	void setHome();
	void showMenu( QModelIndex index );
	void showNoSelectionMenu();

	void cutSlot();
	void copySlot();
	void pasteSlot();
	void removeSlot();
	void newFolderSlot();
	void renameSlot();
	void openWithSlot();
	void openRemotelySlot();
	void openRemotelySlotFinished(QStringList fileNames,QString targetDir);
	void findFileSlot();
	void filePropertiesSlot();

	void setEnabledButtons();
	void changeView();
	void openFile();
	void cancelSearch();

protected:
	void keyPressEvent(QKeyEvent *event);

	Ui::FileSystemWidget *ui;
	PlacesModel * placesModel;

private:
	void init(QString openLocation);
	void initModel();
	void updatePath(QString *path);
	void disconnectModelSignals();
	void connectModelSignals();
	QString remotePlaceName(int i);
	QString findFile(QString searchString);
	void closeEvent(QCloseEvent *closeEvent);

	FileSystemModel * model;

	QAbstractItemView * currentView;
	QModelIndex selectedIndex;
	QString selectedApp;
	bool cut;
	QMenu * menu;
	QMenu * noSelectionMenu;
	QAction * renameAction;
	QAction * newFolderAction;
	QAction * cutAction;
	QAction * copyAction;
	QAction * pasteAction;
	QAction * removeAction;
	QAction * openRemotelyAction;
	QAction * openWithAction;
	QAction * findFileAction;
	QAction * propertiesAction;
	QMutex mutex;
	QProgressDialog * searchProgress;
	bool searching;
	bool standalone;
	QString scrollTo;

	RemoteClientManager * remoteClientManager;
};

#endif // FILESYSTEMWIDGET_H
