#ifndef FILETHREADWIDGET_H
#define FILETHREADWIDGET_H

#include <QDialog>
#include <QProgressBar>
#include <QPushButton>
#include <QLabel>
#include <QGridLayout>
#include <QFileInfo>
#include <QDateTime>
#include <QTimer>
#include <QCloseEvent>

#include "filethread.h"
#include "fileproperties.h"

class FileThreadWidget : public QDialog
{
	Q_OBJECT
public:
	FileThreadWidget(FileThread * fileThread, QWidget *parent = nullptr);
	~FileThreadWidget();

	void start();
	static QString readableSeconds( long long seconds );

protected:
	void closeEvent(QCloseEvent * event);

private slots:
	void updateValues(QFileInfo file, long long totalSize, long long totalCopied, long long currentFileSize, long long currentFileCopied);
	void updateValues();
	void finished();
	void cancel();
	void changedCurrentOperation(FileThread::FileOperation operation);
	void error(FileThread::FileError error, QStringList files);
	void setCurrentFile(QString file);

private:
	void initUi();

	FileThread * fileThread;
	QProgressBar * partialProgressBar;
	QProgressBar * progressBar;
	QLabel * speed;
	QLabel * fileName;
	QLabel * status;
	QPushButton * acceptButton;
	QPushButton * cancelButton;

	QDateTime startTime;
	QTimer * timer;
	long long maxSpeed;
	bool _close;
};

#endif // FILETHREADWIDGET_H
