#include "fontsconfig.h"

FontsConfig::FontsConfig(QWidget *parent ) : QDialog( parent )
{
	setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	init ();
}

void FontsConfig::init()
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	QFont bFont;
	bFont.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );

	QLabel * normalFontLabel = new QLabel(tr("Default font:"), this);
	normalFontLabel->setMinimumWidth( 150 );
	normalFontLabel->setFont( bFont );
	normalFont = new QComboBox( this );
	normalFont->setMinimumWidth( 200 );

	QLabel * boldFontLabel = new QLabel(tr("Bold font:"), this);
	boldFontLabel->setMinimumWidth( 150 );
	boldFontLabel->setFont( bFont );
	boldFont = new QComboBox( this );
	boldFont->setMinimumWidth( 200 );

	QLabel * fontSizeLabel = new QLabel(tr("Default font size:"), this);
	fontSizeLabel->setFont( bFont );
	fontSize = new QSpinBox( this );
	fontSize->setMinimum(6);
	fontSize->setMaximum(18);

	QLabel * appButtonFontSizeLabel = new QLabel(tr("Desktop icons font size:"), this);
	appButtonFontSizeLabel->setFont( bFont );
	appButtonFontSize = new QSpinBox( this );
	appButtonFontSize->setMinimum(6);
	appButtonFontSize->setMaximum(18);

	acceptButton = new QPushButton( tr("Accept"), this);
	acceptButton->setFlat( true );
	acceptButton->setStyleSheet(STANDARDBUTTONSTYLE);
	acceptButton->setMinimumSize( 100, 24 );
	cancelButton = new QPushButton( tr("Cancel"), this);
	cancelButton->setFlat( true );
	cancelButton->setStyleSheet(STANDARDBUTTONSTYLE);
	cancelButton->setMinimumSize( 100, 24 );

	layout->addWidget(normalFontLabel, 1, 0);
	layout->addWidget(normalFont, 1, 1, 1, 5);
	layout->addWidget(boldFontLabel, 2, 0);
	layout->addWidget(boldFont, 2, 1, 1, 5);
	layout->addWidget(fontSizeLabel, 3, 0);
	layout->addWidget(fontSize, 3, 1, 1,5);
	layout->addWidget(appButtonFontSizeLabel, 4, 0);
	layout->addWidget(appButtonFontSize, 4, 1, 1, 5);
	layout->addWidget(acceptButton, 5, 4);
	layout->addWidget(cancelButton, 5, 5);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect(acceptButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	loadSettings();
}

void FontsConfig::loadSettings()
{
	QFontDatabase fontDatabase;
	QStringList fonts = fontDatabase.families();

	normalFont->addItems( fonts );
	normalFont->setCurrentText( Settings::getUserValue("Font", LIBREWORK_FONT).toString() );

	boldFont->addItems( fonts );
	boldFont->setCurrentText( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );

	fontSize->setValue( Settings::getUserValue("FontSize", QString::number(LIBREWORK_FONT_BASE_SIZE)).toInt() );

	appButtonFontSize->setValue( Settings::getUserValue("AppButtonFontSize", QString::number(LISTVIEWITEM_FONT_SIZE)).toInt() );
}

void FontsConfig::saveSettings()
{
	Settings::setUserValue( "Font", normalFont->currentText() );
	Settings::setUserValue( "BoldFont", boldFont->currentText() );
	Settings::setUserValue( "FontSize", QString::number(fontSize->value()) );
	Settings::setUserValue( "AppButtonFontSize", QString::number(appButtonFontSize->value()) );

	QMessageBox::information( this, tr("Restart needed"), tr("The session must be restarted in order to implement changes") );
}

void FontsConfig::accept()
{
	saveSettings();
	setFocus(); // Get out focus from button
}

void FontsConfig::reject()
{
	loadSettings(); // Set user settings values
	setFocus(); // Get out focus from button
}
