#include "iconview.h"

IconView::IconView(QWidget *parent, bool pressedButtons) : QListView(parent)
{
	allowExternalDrops = false;
	this->pressedButtons = pressedButtons;
	delegate = new IconViewItemDelegate;
	setItemDelegate( delegate );
	setAttribute(Qt::WA_TranslucentBackground);
	setViewMode( QListView::IconMode );
	QFont font;
	font.setFamily("arial");
	setFont( font );
	setIconSize( QSize(ICON_SIZE, ICON_SIZE) );
	setGridSize( LISTVIEWITEM_GRID_SIZE );
	setResizeMode( QListView::Fixed );
	setMovement( QListView::Snap );
	setDropIndicatorShown( true );
	setDragDropMode( QAbstractItemView::DragDrop );
}

IconView::~IconView()
{
	delete delegate;
}

void IconView::setExternalDropsAllowed( bool allowExternalDrops )
{
	this->allowExternalDrops = allowExternalDrops;
}

void IconView::setMimeFormat( QString mimeFormat )
{
	this->mimeFormat = mimeFormat;
}

void IconView::setFileSystemMode( bool fileSystemMode )
{
	this->fileSystemMode = fileSystemMode;
	this->setMimeFormat("text/uri-list");
	this->allowExternalDrops = true;
}

void IconView::setSelectionShadow( bool selectionShadow )
{
	delegate->setSelectionShadow( selectionShadow );
}

void IconView::setOrientation(Qt::Orientation orientation)
{
	if ( orientation == Qt::Vertical )
	{
		setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
		setFlow( QListView::LeftToRight );
	}
	if ( orientation == Qt::Horizontal )
	{
		setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
		setFlow( QListView::TopToBottom );
	}
}

void IconView::swapPosition(QModelIndex firstIndex, QModelIndex secondIndex)
{
	QPoint firstPosition = rectForIndex( firstIndex ).topLeft();
	QPoint secondPosition = rectForIndex( secondIndex ).topLeft();
	setPositionForIndex( secondPosition, firstIndex );
	setPositionForIndex( firstPosition, secondIndex );
}

void IconView::mousePressEvent(QMouseEvent *event)
{
	if ( pressedButtons )
	{
		delegate->setPressed( event->pos() );
		repaint();
	}
	QListView::mousePressEvent( event );
}

void IconView::mouseReleaseEvent(QMouseEvent *event)
{
	if ( pressedButtons )
	{
		delegate->setPressed( QPoint(-1, -1) );
		repaint();
	}
	QListView::mouseReleaseEvent( event );
}

void IconView::dragEnterEvent(QDragEnterEvent* event)
{
	DragDrop::dragEnterEvent( event, mimeFormat );
}

void IconView::dragMoveEvent(QDragMoveEvent* e)
{
	DragDrop::dragMoveEvent( e, mimeFormat );
}

void IconView::dropEvent(QDropEvent* e)
{
	QPair<QStringList,QModelIndex> ret = DragDrop::dropEvent( e, this );
	if ( ret.first.count() > 0 )
	{
		emit dropFiles( ret.second, ret.first, e );
	}
}

void IconView::startDrag(Qt::DropActions supportedActions)
{
	DragDrop::startDrag( supportedActions, this, fileSystemMode );
}
