#ifndef ICONVIEW_H
#define ICONVIEW_H

#include <QListView>
#include <QListWidgetItem>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QFileSystemModel>
#include <QMouseEvent>
#include <QMessageBox>
#include <QApplication>
#include <QModelIndex>
#include <QMimeData>
#include <QDrag>
#include <QDragEnterEvent>
#include <QStringList>
#include <QUrl>

#include "constants.h"
#include "settings.h"
#include "iconviewitemdelegate.h"
#include "dragdrop.h"

class IconView : public QListView
{
	Q_OBJECT
public:
	explicit IconView(QWidget *parent = 0, bool pressedButtons = false);
	~IconView();

	void setOrientation(Qt::Orientation orientation);
	void setPressedButtons( bool pressedButtons ) { this->pressedButtons = pressedButtons; }
	void setExternalDropsAllowed( bool allowExternalDrops );
	void setMimeFormat( QString mimeFormat );
	void setFileSystemMode( bool fileSystemMode );
	void setSelectionShadow( bool selectionShadow );
	void swapPosition(QModelIndex firstIndex, QModelIndex secondIndex);

signals:
	void dropFiles(QModelIndex target, QStringList fileList, QDropEvent * e);

protected:
	void mousePressEvent(QMouseEvent *event);
	void mouseReleaseEvent(QMouseEvent *event);
	void dragEnterEvent(QDragEnterEvent* event);
	void dragMoveEvent(QDragMoveEvent* e);
	void dropEvent(QDropEvent* e);
	void startDrag(Qt::DropActions supportedActions);
	QString mimeFormat;
	bool fileSystemMode;

private:
	IconViewItemDelegate * delegate;
	bool pressedButtons;
	bool allowExternalDrops;
};

#endif // ICONVIEW_H
