#include "lwclient.h"

#include <QTimer>

static LWClient * _instance = 0;

LWClient::LWClient()
{
	tcpSocket = 0;
	_isConnected = false;
	_isAuthenticated = false;
}

LWClient *  LWClient::instance()
{
	if ( _instance == 0 )
	{
		_instance = new LWClient();
	}
	return _instance;
}

void LWClient::close(bool sendQuit)
{
	if ( _isConnected  && sendQuit )
	{
		QString message = "quit\n";
		sendMessage( message );
	}
	LWSocket::closeSocket( tcpSocket );
	_isConnected = false;
	_isAuthenticated = false;
}

bool LWClient::connect(QString _address, int _port)
{
	if ( _isConnected )
	{
		return true;
	}

	int port = _port;
	if ( port == -1 )
	{
		port = Settings::getValue( "Server/LibreworkServerPort" ).toInt();
	}
	QString address = _address;
	if ( address.isEmpty() )
	{
		address = Settings::getValue( "Server/LibreworkServerAddress" );
	}

	qDebug("Connecting to %s:%d", address.toUtf8().data(), port);

	tcpSocket = LWSocket::createRemoteSocket( address, port );

	if ( tcpSocket < 0)
	{
		qDebug("Cannot create client socket (error %d)", tcpSocket);
		_isConnected = false;
	} else {
		qDebug("Connected");
		_isConnected = true;
	}

    return _isConnected;
}

bool LWClient::authenticate(QString username, QString password)
{
	if ( _isAuthenticated )
	{
		return true;
	}

	if ( _isConnected )
	{
		// Authentication
		QString message = "login?" + username + "&&" + password + "\n";
		QString response;
		bool status = LWSocket::sendMessage( tcpSocket, message );
		if ( status )
		{
			response = LWSocket::receiveMessage( tcpSocket );
		}

		if ( ! status || response.startsWith( "Error" ) || response.isEmpty() )
		{
			_isAuthenticated = false;
		} else {
			_isAuthenticated = true;
		}
	} else {
		_isAuthenticated = false;
	}
	return _isAuthenticated;
}

QByteArray LWClient::sendMessageByteArray(QString message)
{
	mutex.lock();

	QByteArray response;

	if ( _isConnected )
	{
		if ( LWSocket::sendMessage( tcpSocket, QString( "_byteArray_" + message).toUtf8().data()) )
		{
			QByteArray resSize = LWSocket::receiveByteArray( tcpSocket, 4 );

			if ( resSize.count() == 4 )
			{
				//Settings::logInfo( QString("resSize size: %1").arg(resSize.size()) );
				int responseSize = 0;

				responseSize |= ( (unsigned char) resSize[0] << 24 );
				responseSize |= ( (unsigned char) resSize[1] << 16 );
				responseSize |= ( (unsigned char) resSize[2] <<  8 );
				responseSize |= ( (unsigned char) resSize[3] );

				//Settings::logInfo( QString("Response size: %1").arg(responseSize) );

				QTimer timer;
				timer.start(TIMEOUT * 1000);

				while (response.size() < responseSize && timer.isActive() )
				{
					response += LWSocket::receiveByteArray( tcpSocket );
				}
			}
		}
	}

	mutex.unlock();

	return response;
}

QByteArray LWClient::sendMessageByteArray(QString command, QStringList parameters)
{
	QString message = command;
	if ( parameters.count() > 0 )
	{
		message += "?" + parameters.join("&");
	}
	return sendMessageByteArray( message );
}

QString LWClient::sendMessage(QString message)
{
	qDebug("Sending message: '%s'", message.toUtf8().data());
	mutex.lock();

	QString response;

	if ( _isConnected )
	{
		if ( LWSocket::sendMessage( tcpSocket, message + "\n") )
		{
			response = LWSocket::receiveMessage( tcpSocket );
			qDebug("Response: %s", response.toUtf8().data());
		} else {
			qDebug("Cannot send message '%s'", message.toUtf8().data());
		}
		response = response.trimmed();
	}

	mutex.unlock();

	return response;
}

long long LWClient::sendData(QByteArray data)
{
	if ( LWSocket::sendByteArray( tcpSocket, data ) )
	{
		return data.size();
	} else {
		return 0;
	}
}

QString LWClient::sendMessage(QString command, QStringList parameters)
{
	QString message = command;
	if ( parameters.count() > 0 )
	{
		message += "?" + parameters.join("&");
	}
	return sendMessage( message );
}
