#include "menu.h"
#include "ui_menu.h"

Menu::Menu(QWidget *parent) :
	QWidget(parent, Qt::FramelessWindowHint | Qt::CustomizeWindowHint | Qt::Popup),
	ui(new Ui::Menu)
{
	currentAppItem = 0;
	ui->setupUi(this);
	ui->usernameLabel->setText( Settings::getUserName() );
	ui->filterListWidget->setStyleSheet("QListWidget {"
										"	background: " TABCOLOR ";"
										"	color: " FONTCOLOR ";"
										"	font-size: " + QString::number((LIBREWORK_FONT_BASE_SIZE * 4) / 3) + "px;"
										"	selection-background-color: " TABSELECTIONCOLOR ";"
										"}"
										);
	ui->applicationsListWidget->setStyleSheet("QListWidget {"
										"	background: " CONFIGURATIONBACKGROUNDCOLOR ";"
										"	color: " CONFIGURATIONFONTCOLOR ";"
										"	font-size: " + QString::number((LIBREWORK_FONT_BASE_SIZE * 4) / 3) + "px;"
										"	selection-background-color: " TABSELECTIONCOLOR ";"
										"}"
										);

	ui->filterListWidget->setIconSize(QSize(MENU_ICON_SIZE, MENU_ICON_SIZE));
	ui->applicationsListWidget->setContextMenuPolicy(Qt::CustomContextMenu);

	this->setStyleSheet("Menu {"
						"	background: " TABCOLOR ";"
						"}"
						);

	updateFilters();

	connect(ui->searchLineEdit, SIGNAL(focusOut()), this, SLOT(setFocus()));
	connect(ui->searchLineEdit, SIGNAL(textChanged(QString)), this, SLOT(searchTextChanged(QString)));
	connect(ui->filterListWidget, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(setApplications(QListWidgetItem*)));
	connect(ui->applicationsListWidget, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(launchApplication(QListWidgetItem*)));
	connect(ui->applicationsListWidget, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(applicationMenuRequested(QPoint)));
}

Menu::~Menu()
{
	delete ui;
}

QListWidgetItem * Menu::createItem(QIcon icon, QString text, QString id, bool checkGroup)
{
	if ( checkGroup )
	{
		int appCount = Application::applicationListByGroup( id ).count();
		if ( appCount == 0 )
		{
			return 0;
		}
	}
	QListWidgetItem * item = new QListWidgetItem(icon, text);
	item->setTextAlignment(Qt::AlignVCenter);
	item->setData(Qt::UserRole, id);
	item->setSizeHint(QSize(MENU_ICON_SIZE + ICONMARGIN, MENU_ICON_SIZE + ICONMARGIN));
	return item;
}

void Menu::updateFilters()
{
	ui->filterListWidget->clear();

	favorites = createItem(QIcon(":/icons/groups/favorites.png"), tr("Favorites"), "favorites");
	ui->filterListWidget->addItem( favorites );

	QListWidgetItem * all = createItem(QIcon(":/icons/groups/all.png"), tr("All apps"), "all");
	ui->filterListWidget->addItem( all );

	searchResults = createItem(QIcon(":/icons/groups/search.png"), tr("Search results"), "search");
	ui->filterListWidget->addItem( searchResults );

	separator = new QListWidgetItem();
	QFrame * separatorFrame = new QFrame();
	separatorFrame->setFrameShape(QFrame::HLine);
	separatorFrame->setStyleSheet("QFrame"
								  "{"
								  "		color: " FONTCOLOR ";"
								  "}"
								  );
	separator->setFlags(Qt::NoItemFlags);
	ui->filterListWidget->addItem( separator );
	ui->filterListWidget->setItemWidget( separator, separatorFrame );

	QStringList groupIds = Application::applicationGroupIdsList();
	QStringList groups = Application::applicationGroupsList();

	if ( groupIds.count() == groups.count() ) // Malformed file detection!
	{
		for ( int i = 0; i < groupIds.count(); i++ )
		{
			QListWidgetItem * item = createItem(Application::applicationGroupIcon(groupIds[i]), groups[i], groupIds[i], true);
			if ( item != 0 )
			{
				ui->filterListWidget->addItem( item );
			}
		}
	}

	QListWidgetItem * other = createItem(QIcon(":/icons/groups/other.png"), tr("Other"), "other", true);
	if ( other != 0 )
	{
		ui->filterListWidget->addItem( other );
	}

	setApplications( favorites );
}

void Menu::searchTextChanged(QString)
{
	setApplications(searchResults);
}

QList<Application*> Menu::searchApplicationList(QString searchString)
{
	QList<Application*> ret;
	QList<Application*> all = Application::applicationListByGroup( "all" );
	for ( int i = 0; i < all.count(); i++ )
	{
		if ( all[i]->getName().contains( searchString, Qt::CaseInsensitive ) || all[i]->getDescription().contains( searchString, Qt::CaseInsensitive ) )
		{
			ret += all[i];
		}
	}
	return ret;
}

void Menu::setApplications(QListWidgetItem* item)
{
	ui->filterListWidget->setCurrentItem( item );

	apps.clear();
	ui->applicationsListWidget->clear();

	ui->filterListWidget->setItemSelected( item, true );
	if ( item->data(Qt::UserRole) == "favorites" )
	{
		apps += Application::favoriteApplicationList();
	} else if ( item->data(Qt::UserRole) == "search" ) {
		if ( ! ui->searchLineEdit->text().isEmpty() )
		{
			apps = searchApplicationList( ui->searchLineEdit->text() );
		}
	} else {
		apps = Application::applicationListByGroup( item->data(APP_NAME).toString() );
	}
	for ( int i = 0; i < apps.count(); i++ )
	{
		QListWidgetItem * item = createItem(apps[i]->getIcon(), apps[i]->getName(), apps[i]->getId());
		ui->applicationsListWidget->addItem( item );
	}
}

void Menu::applicationMenuRequested(QPoint point)
{
	currentAppItem = ui->applicationsListWidget->currentItem();
	if ( currentAppItem != 0 )
	{
		QMenu * menu = new QMenu(ui->applicationsListWidget);
		if ( ui->filterListWidget->currentItem() == favorites )
		{
			menu->addAction(tr("Remove from favorites"), this, SLOT(removeFromFavorites()));
		} else {
			menu->addAction(tr("Add to favorites"), this, SLOT(addToFavorites()));
		}
		menu->exec( this->mapToGlobal(point) );
	}
}

void Menu::addToFavorites()
{
	if ( currentAppItem != 0 )
	{
		QStringList favoritesStringList = Settings::getFavoriteApplications();
		QString app = currentAppItem->data(APP_NAME).toString();
		if ( ! favoritesStringList.contains( app ) )
		{
			favoritesStringList.append( app );
			Settings::setFavoriteApplications( favoritesStringList );
		}
		setApplications( favorites );
		emit favoritesChanged();
	}
}

void Menu::removeFromFavorites()
{
	if ( currentAppItem != 0 )
	{
		QStringList favoritesStringList = Settings::getFavoriteApplications();
		QString app = currentAppItem->data(APP_NAME).toString();
		if ( favoritesStringList.contains( app ) )
		{
			 favoritesStringList.removeAll( app );
			 Settings::setFavoriteApplications( favoritesStringList );
		}
		setApplications( favorites );
		emit favoritesChanged();
	}
}

void Menu::launchApplication(QListWidgetItem * item)
{
	int index = ui->applicationsListWidget->row( item );
	if ( index >= 0 && index < apps.count() )
	{
		apps[index]->launch();
	}
	hide();
}

void Menu::focusOutEvent(QFocusEvent * event)
{
	if ( ! ui->searchLineEdit->hasFocus() )
	{
		bool childFocus = false;
		for ( int i = 0; i < children().count(); i++ )
		{
			if  ( children()[i]->isWidgetType() )
			{
				if ( ((QWidget*)children()[i])->hasFocus() )
				{
					childFocus = true;
				}
			}
		}
		if ( childFocus )
		{
			setFocus();
		} else {
			hide();
			setApplications( favorites );
		}
	}
	QWidget::focusOutEvent( event );
}

void Menu::focusInEvent(QFocusEvent * event)
{
	QWidget::focusInEvent( event );
}

void Menu::keyPressEvent(QKeyEvent * event)
{
	switch ( event->key() )
	{
	case Qt::Key_Up:
		goUp();
		break;
	case Qt::Key_Down:
		goDown();
		break;
	case Qt::Key_Left:
		goLeft();
		break;
	case Qt::Key_Right:
		goRight();
		break;
	case Qt::Key_Enter:
	case Qt::Key_Return:
		if ( ui->applicationsListWidget->currentItem() != 0 )
		{
			launchApplication( ui->applicationsListWidget->currentItem() );
		}
		break;
	case Qt::Key_Escape:
		hide();
		setApplications( favorites );
		break;
	default:
		ui->searchLineEdit->keyPressEvent( event );
	}
}

void Menu::keyReleaseEvent(QKeyEvent * event)
{
	switch( event->key() )
	{
	case Qt::Key_Up:
	case Qt::Key_Down:
	case Qt::Key_Left:
	case Qt::Key_Right:
	case Qt::Key_Enter:
	case Qt::Key_Return:
	case Qt::Key_Escape:
		// Do nothing
		break;
	default:
		ui->searchLineEdit->keyReleaseEvent( event );
	}
}

void Menu::goUp()
{
	if ( ui->applicationsListWidget->currentItem() != 0 )
	{
		int row = ui->applicationsListWidget->currentRow();
		if ( row > 0 )
		{
			ui->applicationsListWidget->setCurrentRow( row - 1 );
		}
	} else {
		if ( ui->filterListWidget->currentItem() != 0 )
		{
			int row = ui->filterListWidget->currentRow();
			if ( row > 0 )
			{
				ui->filterListWidget->setCurrentRow( row - 1 );
				QListWidgetItem * item = ui->filterListWidget->currentItem();
				setApplications( item );
			}
		}
	}
}

void Menu::goDown()
{
	if ( ui->applicationsListWidget->currentItem() != 0 )
	{
		int row = ui->applicationsListWidget->currentRow();
		if ( row < ui->applicationsListWidget->count() - 1 )
		{
			ui->applicationsListWidget->setCurrentRow( row + 1 );
		}
	} else {
		if ( ui->filterListWidget->currentItem() != 0 )
		{
			int row = ui->filterListWidget->currentRow();
			if ( row < ui->filterListWidget->count() - 1 )
			{
				ui->filterListWidget->setCurrentRow( row + 1 );
				QListWidgetItem * item = ui->filterListWidget->currentItem();
				setApplications( item );
			}
		}
	}
}

void Menu::goLeft()
{
	if ( ui->applicationsListWidget->currentItem() != 0 )
	{
		ui->applicationsListWidget->setCurrentItem( 0 );
	}
}

void Menu::goRight()
{
	if ( ui->applicationsListWidget->currentItem() == 0 )
	{
		ui->applicationsListWidget->setCurrentRow( 0 );
		ui->applicationsListWidget->setItemSelected( ui->applicationsListWidget->currentItem(), true );
	}
}

int Menu::shutDown(int command)
{
	QString commandString;
	switch(command)
	{
	case COMMAND_SHUTDOWN:
		commandString = "PowerOff";
		break;
	case COMMAND_REBOOT:
		commandString = "Reboot";
		break;
	case COMMAND_SUSPEND:
		commandString = "Suspend";
		break;
	case COMMAND_HIBERNATE:
		commandString = "Hibernate";
		break;
	default:
		commandString = "CloseSession";
		break;
	}

	if ( command == COMMAND_CLOSE_SESSION )
	{
		return 0;
	}

#ifdef Q_OS_LINUX
	QDBusInterface logind{"org.freedesktop.login1", "/org/freedesktop/login1", "org.freedesktop.login1.Manager", QDBusConnection::systemBus()};
	const auto message = logind.callWithArgumentList(QDBus::Block, "Can" + commandString, {});
	QDBusPendingReply< QString > canPowerOff = message;
	Q_ASSERT(canPowerOff.isFinished());
	if (canPowerOff.isError()) {
		const auto error = canPowerOff.error();
		qWarning().noquote()
				<< QDBusInterface::tr("Asynchronous call finished with error: %1 (%2)")
				   .arg(error.name(), error.message());
		return EXIT_FAILURE;
	}
	if (canPowerOff.value() == "yes") {
		QDBusPendingReply<> shutDown = logind.callWithArgumentList(QDBus::Block, commandString, {true, });
		Q_ASSERT(shutDown.isFinished());
		if (shutDown.isError()) {
			const auto error = shutDown.error();
			qWarning().noquote()
					<< QDBusInterface::tr("Asynchronous call finished with error: %1 (%2)")
					   .arg(error.name(), error.message());
			return EXIT_FAILURE;
		}
	} else {
		qCritical().noquote()
				<< QCoreApplication::translate("poweroff", "Can't power off: CanPowerOff() result is %1")
				   .arg(canPowerOff.value());
		return EXIT_FAILURE;
	}
#endif
	return 0;
}
