#ifndef MENU_H
#define MENU_H

#include <QWidget>
#include <QAction>
#include <QListWidgetItem>
#include <QFocusEvent>
#include <QKeyEvent>
#include <QMenu>

#ifdef Q_OS_LINUX
#include <QtDBus/QDBusInterface>
#include <QtDBus/QDBusPendingReply>
#include <QtDBus/QDBusConnection>
#endif

#include "application.h"
#include "constants.h"

#define COMMAND_SHUTDOWN 0
#define COMMAND_REBOOT 1
#define COMMAND_SUSPEND 2
#define COMMAND_HIBERNATE 3
#define COMMAND_CLOSE_SESSION 4

#define APP_NAME Qt::UserRole

namespace Ui {
class Menu;
}

class Menu : public QWidget
{
	Q_OBJECT

public:
	explicit Menu(QWidget *parent = 0);
	~Menu();

public slots:
	void updateFilters();

private slots:
	void applicationMenuRequested(QPoint point);
	void addToFavorites();
	void removeFromFavorites();
	void setApplications(QListWidgetItem *item);
	void launchApplication(QListWidgetItem *item);
	void searchTextChanged(QString);

	void focusOutEvent(QFocusEvent * event);
	void focusInEvent(QFocusEvent * event);
	void keyPressEvent(QKeyEvent * event);
	void keyReleaseEvent(QKeyEvent * event);

	int shutDown(int command);

	void goUp();
	void goDown();
	void goLeft();
	void goRight();

signals:
	void favoritesChanged();

private:
	QListWidgetItem * createItem(QIcon icon, QString text, QString id, bool checkGroup = false);
	QList<Application*> searchApplicationList(QString searchString);

	Ui::Menu *ui;
	QListWidgetItem * favorites;
	QListWidgetItem * searchResults;
	QListWidgetItem * currentAppItem;
	QListWidgetItem * separator;
	QList<Application*> apps;
};

#endif // MENU_H
