#include "mime.h"

void Mime::updateDesktopFiles()
{
	if ( LWClient::instance()->isConnected() )
	{
#ifdef Q_OS_LINUX
		QFile::remove( MIME_CACHE_FILE );
		QHash<QString,QStringList> mimeCacheData;
		QList<Application *> appList = Application::applicationList();
		for ( int i = 0; i < appList.count(); i++ )
		{
			QString desktopFile = addDesktopFile( appList[i] );
			addApplication( appList[i], desktopFile, mimeCacheData );
		}
		writeMimeCacheFile( mimeCacheData );
#endif
	}
}

QString Mime::addDesktopFile(Application * application)
{
#ifdef Q_OS_UNIX
	QString desktopFile = MIME_CACHE_DIR + QDir::separator() + application->getName() + ".desktop";
	QFile file( desktopFile );
	if ( file.open( QIODevice::WriteOnly ) )
	{
		file.write("[Desktop Entry]");
		file.write("Type=Application");
		file.write( QString( "Name=" + application->getName() ).toUtf8() );
		file.write( QString( "MimeType=" + application->getMimeTypes().join(";") + ";" ).toUtf8() );
		QPixmap icon = application->getIcon().pixmap( 128, 128 );
		QString iconFileName = application->getName() + ".png";
		icon.save( MIME_CACHE_DIR + iconFileName );
		file.write( QString( "Icon=" + iconFileName ).toUtf8() );
		file.write( QString( "Categories=" + application->getGroups().join(";") + ";" ).toUtf8() );
		file.write("StartupNotify=true");
		file.write("Terminal=false");
		//QString portString = QString::number( LWDesktopServer::instance()->port() );
		//file.write( QString( "Exec=libreworkdesktoplauncher.sh " + portString + " " + application->getId() + " %f" ).toUtf8() );
		file.close();
		return desktopFile;
	}
#else
	Q_UNUSED(application);
#endif
	return QString();
}

QHash<QString,QStringList> Mime::readMimeCacheFile()
{
	QHash<QString,QStringList> fileContents;
#ifdef Q_OS_UNIX
	QFile file( MIME_CACHE_FILE );
	if ( file.exists() )
	{
		file.open( QIODevice::ReadOnly );
		QString line = QString(file.readLine()).simplified();
		if ( ! line.startsWith("[") && ! line.startsWith(";") && ! line.startsWith("#") && line.size() > 0 )
		{
			if ( line.contains("=") )
			{
				QString key = line.section( '=', 0, 0 );
				QStringList values = line.section( '=', 1, 1 ).split( ';' );
				fileContents[ key ] = values;
			}
		}
	}
#endif
	return fileContents;
}

void Mime::writeMimeCacheFile( QHash<QString,QStringList> mimeCache )
{
#ifdef Q_OS_UNIX
	QFile file( MIME_CACHE_FILE );
	file.open( QIODevice::WriteOnly );
	file.write( "[MIME Cache]\n" );

	QHashIterator<QString, QStringList> i(mimeCache);
	while (i.hasNext())
	{
		i.next();
		if ( ! i.value().count() == 0 )
		{
			file.write( QString( i.key() + "=" + i.value().join(';') + ";" ).toUtf8() );
		}
	}
	file.close();
#endif
}

void Mime::addApplication(Application * application, QString desktopFile, QHash<QString,QStringList> & mimeCacheData)
{
	QStringList mimeTypes = application->getMimeTypes();
	for ( int i = 0; i < mimeTypes.count(); i++ )
	{
		QFileInfo desktopFileInfo( desktopFile );
		mimeCacheData[ mimeTypes[i] ] += desktopFileInfo.fileName();
	}
}
