#include "appbutton.h"

AppButton::AppButton (Application * app, QString fontColor, QWidget * parent ) : QPushButton( parent )
{
	setFocusPolicy(Qt::NoFocus);
	this->fontColor = fontColor;
	this->app = app;
	this->parent = parent;
	widget = 0;
	substract = 0;
	highlight = false;
	connect(this, SIGNAL(clicked()), this, SLOT(buttonClicked()));
	setFixedSize( buttonSize() );

	if (app->getName().isEmpty())
	{
		text = tr("Unkonwn");
	} else {
		text = app->getName();
	}

	int numSpaces = text.count(" ");
	if ( numSpaces < 2 )
	{
		text.replace( " ", "\n" );
	} else {
		int bestPos = text.length() / 2;
		int pos = text.length();

		for ( int i = 0; i < numSpaces; i++ )
		{
			int p = text.indexOf( " ", i );
			int newPos = abs( bestPos - p );
			if ( newPos < pos )
			{
				pos = p;
			}
		}

		text.replace( pos, 1, "\n" );
	}

	icon = app->getIcon();
}

AppButton::AppButton (QIcon icon, QString name, QWidget * widget, QString fontColor, QWidget * parent ) : QPushButton( parent )
{
	this->fontColor = fontColor;
	connect(this, SIGNAL(clicked()), this, SLOT(buttonClicked()));
	setFixedSize( buttonSize() );
	this->icon = icon;
	this->text = name;
	this->widget = widget;
	this->parent = parent;
	app = 0;
	substract = 0;
	highlight = false;
}

QSize AppButton::buttonSize()
{
	return QSize( ICON_SIZE + 2 * HMARGIN, ICON_SIZE + 2 * Settings::getUserValue("AppButtonFontSize", QString::number(APPBUTTON_FONT_SIZE)).toInt() + 2 * SEPARATION + 2 * VMARGIN );
}

void AppButton::buttonClicked()
{
	if (app == 0)
		emit appButtonClicked(widget);
	else
		emit appButtonClicked(app);
}

bool AppButton::event(QEvent *event)
{
	return QPushButton::event( event );
}

void AppButton::enterEvent(QEvent * event)
{
	QPushButton::enterEvent( event );
	if ( parent->cursor().shape() != Qt::WaitCursor )
	{
		highlight = true;
#ifdef Q_OS_WIN
		forceRepaint();
#else
		repaint();
#endif
	}
}

void AppButton::leaveEvent(QEvent * event)
{
	if ( highlight )
	{
		highlight = false;
#ifdef Q_OS_WIN
		forceRepaint();
#else
		repaint();
#endif
	}
	QPushButton::leaveEvent( event );
}
void AppButton::mousePressEvent(QMouseEvent * event)
{
	QPushButton::mousePressEvent( event );
	if ( parent->cursor().shape() != Qt::WaitCursor )
	{
		substract = 5;
#ifdef Q_OS_WIN
		forceRepaint();
#else
		repaint();
#endif
	}
}

void AppButton::mouseReleaseEvent(QMouseEvent * event)
{
	if (substract != 0)
	{
		substract = 0;
#ifdef Q_OS_WIN
		forceRepaint();
#else
		repaint();
#endif
	}
	QPushButton::mouseReleaseEvent( event );
}

void AppButton::forceRepaint()
{
	QSize _size = size();
	setFixedSize(size().width() - 1, size().height() - 1);
	setFixedSize(_size);
}

void AppButton::paintEvent ( QPaintEvent * e)
{
	Q_UNUSED(e);
	QPainter p(this);

	if ( ! icon.isNull() )
	{
		QPoint upLeft(HMARGIN + substract, VMARGIN + substract);
		QPoint downRight(HMARGIN + ICON_SIZE - substract, VMARGIN + ICON_SIZE - substract);
		QRect rectangle (upLeft, downRight);
		QPixmap pixmap = icon.pixmap(ICON_SIZE - 2 * substract, ICON_SIZE - 2 * substract);
		if (highlight)
		{
			QGraphicsScene scene;
			QGraphicsPixmapItem pixmapItem(pixmap);
			QGraphicsColorizeEffect effect;
			QColor hColor(255, 255, 128, 32);
			effect.setColor(hColor);
			pixmapItem.setGraphicsEffect( & effect );
			scene.addItem(&pixmapItem);
			scene.render(&p, rectangle);
		} else {
			p.drawPixmap(rectangle, pixmap);
		}
	}

	int appButtonFontSize = Settings::getUserValue("AppButtonFontSize", QString::number(APPBUTTON_FONT_SIZE)).toInt();
	QFont sansFont;//("Helvetica", FONT_SIZE);
	sansFont.setPixelSize(appButtonFontSize);
	p.setPen( QColor( fontColor ) );
	//p.setPen(QColor(40, 40, 40));
	p.setFont(sansFont);
	QPoint upLeft(0, VMARGIN + ICON_SIZE + SEPARATION);
	QPoint downRight(2 * HMARGIN + ICON_SIZE, VMARGIN + ICON_SIZE + 2 * SEPARATION + 2 * appButtonFontSize);
	QRect rectangle (upLeft, downRight);
	p.setOpacity(0.85);
	//p.fillRect ( rectangle, QBrush(QColor(BACKGROUNDCOLOR)) );
	p.drawText(rectangle, Qt::AlignTop | Qt::AlignHCenter, text);
}
