#ifndef DESKTOP_H
#define DESKTOP_H

#include <QTextStream>

#include <QApplication>

#include <QTabWidget>
#include <QDesktopWidget>
#include <QPushButton>
#include <QIcon>
#include <QProcess>
#include <QAbstractEventDispatcher>
#include <QVBoxLayout>
#include <QList>
#include <QTimer>
#include <QSize>
#include <QPoint>
#include <QTimer>
#include <QMovie>
#include <QLabel>
#include <QWindow>
#include <QGuiApplication>
#include <QMap>
#include <QFileDialog>
#include <QMessageBox>
#include <QFile>

#include "constants.h"
#include "tabbar.h"
#include "settings.h"
#include "lwclient.h"
#include "appserver.h"
#include "sessionmanager.h"
#include "threadbutton.h"
#include "initscreen.h"
#include "tabbar.h"
#include "winmanager.h"

#ifdef Q_OS_LINUX
#include "x11info.h"
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#endif
#ifdef Q_OS_WIN
#include <WtsApi32.h>
#endif
class WinManager;
class TabBar;

class Desktop : public QObject
{
	Q_OBJECT

	public:
		Desktop(QString desktopUsername = QString::null, QString desktopPassword = QString::null);
		~Desktop();

		static QSize getDesktopSize();
		long long getDesktopWinId();

	signals:
		void fileOpened();
		void exiting();

	public slots:
		void exitApp();

	private slots:
		void launchApplication(AppInstance * app);
		void openFile(QString file, QString application = QString::null);
		/* void openMime(QString location, QString mime); */
		void timeout();
        void resize(int screen = -1);
		void clientConnected();
		void setSelected(int index);
		void loadedFinished();
		void setThreadWidgetName( int index );
        void setThreadWidgetTitle( long long window );
        bool connectToServer();
		void moveToBack();

        void mapWindow(long long window);
        void mapNotify(long long window);
        void unmapWindow(long long window);
        void titleChanged(long long window, QString newTitle);
		void mapWidget(QWidget * widget, bool force = false);

        void closeButtonClicked(long long window);
        void killButtonClicked(long long window);

	private:
		SessionManager * sessionManager;
		WinManager * manager;

		QList<Q_PID> pids;
		LWClient * lwClient;

		TabBar * tabBar;

		QTimer timer;
		QIcon currentIcon;
		QString currentName;
		QProcess * currentProcess;
		QMap<Q_PID,QProcess*> process;
		QString launchingRemote;
};

#endif
