#include "loginscreen.h"
#include "ui_loginscreen.h"

#include <QMessageBox>
#include <QDesktopWidget>
#include <QFile>

LoginScreen::LoginScreen(QWidget *parent) : QWidget(parent), ui(new Ui::LoginScreen)
{
	QIcon icon(":/icons/other/librework.icns");
	setWindowIcon( icon );
	WinManager::setRootWindowBackgroundColor( QColor(BACKGROUNDCOLOR) );
	manager = 0;
	errorShown = false;

	ui->setupUi(this);

	qApp->setStyleSheet("Widget{background-image: url(:/icons/other/loginscreen.png);}");
	setWindowTitle(tr("Librework Desktop"));

	setWindowFlags( Qt::CustomizeWindowHint );
	QSize desktopSize = QApplication::desktop()->size();
	QSize widgetSize = size();
	move( (desktopSize.width() - widgetSize.width() ) / 2, (desktopSize.height() - widgetSize.height()) / 2 );

	launchRemovableManagerProcess();

	serversChanged();
	connect(ui->serverLineEdit, SIGNAL(returnPressed()), this, SLOT(on_loginButton_clicked()));
	setShutdownMenu();
	connect(ui->powerOffButton, SIGNAL(customContextMenuRequested(QPoint)), ui->powerOffButton, SLOT(showMenu()));

	/*
	// Start openbox wm
	QStringList openBoxArgs;
	openBoxArgs += "--replace";
	openBoxArgs += "--config-file";
	openBoxArgs += SHAREDIR "rc.xml";
	QProcess::startDetached("openbox", openBoxArgs);
	*/

	ui->loginLineEdit->setText( Settings::getValue("LastUsername") );

	if ( ! ui->loginLineEdit->text().isEmpty() )
	{
		QTimer::singleShot( 0, ui->passwordLineEdit, SLOT(setFocus()));
	}  else {
		QTimer::singleShot( 0, ui->loginLineEdit, SLOT(setFocus()));
	}
}

LoginScreen::~LoginScreen()
{
	if ( removableManagerProcess != 0 )
	{
		removableManagerProcess->kill();
		removableManagerProcess->waitForFinished();
		delete removableManagerProcess;
	}

	if ( manager != 0 )
	{
		delete manager;
	}
	delete ui;
}

void LoginScreen::forceTranslations()
{
	QString ok = tr("OK");
	Q_UNUSED(ok);
	QString cancel = tr("Cancel");
	Q_UNUSED(cancel);
}

void LoginScreen::launchRemovableManagerProcess()
{
	if ( QFile::exists("/usr/bin/removablemanager") )
	{
		removableManagerProcess = new QProcess();
		QStringList arguments;
		arguments += "--minimized";
		removableManagerProcess->start("/usr/bin/removablemanager", arguments);
	}
}

void LoginScreen::serversChanged()
{
	ui->serverLineEdit->clear();
	ui->serverLineEdit->setText( Settings::getDesktopServer() );
}

void LoginScreen::on_loginButton_clicked()
{
	Session::launch( ui->loginLineEdit->text(), ui->passwordLineEdit->text(), this, manager );
}

void LoginScreen::socketError(QString socketError)
{
    finished();
    if ( ! errorShown )
    {
        QMessageBox::critical(this, tr("Network error"), socketError);
        errorShown = true;
    }
}

void LoginScreen::finished()
{
	finished(0);
}

void LoginScreen::finished(int)
{
	bool ret = Session::closeSession();
	Q_UNUSED( ret );

	show();
	activateWindow();
	raise();
	smbUmountAll();
}

void LoginScreen::on_passwordLineEdit_returnPressed()
{
    on_loginButton_clicked();
}

void LoginScreen::on_loginLineEdit_returnPressed()
{
    on_loginButton_clicked();
}

void LoginScreen::on_powerOffButton_clicked()
{
}

void LoginScreen::setShutdownMenu()
{
	QMenu * menu = new QMenu;
	menu->addAction(tr("Power Off"), this, SLOT(shutDown()));
	menu->addAction(tr("Reboot"), this, SLOT(reboot()));
	menu->addAction(tr("Suspend"), this, SLOT(suspend()));
	menu->addAction(tr("Hibernate"), this, SLOT(hibernate()));
	ui->powerOffButton->setMenu( menu );
}

void LoginScreen::shutDown()
{
	QProcess shutdown;
#ifdef Q_OS_LINUX
	shutdown.startDetached("/sbin/shutdown -P now");
#endif
#ifdef Q_OS_WIN
	shutdown.startDetached("shutdown -s -t 0");
#endif
}

void LoginScreen::reboot()
{
	QProcess shutdown;
#ifdef Q_OS_LINUX
	shutdown.startDetached("/sbin/shutdown -r now");
#endif
#ifdef Q_OS_WIN
	shutdown.startDetached("shutdown -r -t 0");
#endif
}

void LoginScreen::suspend()
{
	QProcess shutdown;
#ifdef Q_OS_LINUX
	shutdown.startDetached("/bin/systemctl suspend");
#endif
#ifdef Q_OS_WIN
	shutdown.startDetached("rundll32.exe powrprof.dll,SetSuspendState Standby");
#endif
}

void LoginScreen::hibernate()
{
	QProcess shutdown;
#ifdef Q_OS_LINUX
	shutdown.startDetached("/bin/systemctl hibernate");
#endif
#ifdef Q_OS_WIN
	shutdown.startDetached("rundll32.exe powrprof.dll,SetSuspendState Hibernate");
#endif
}
