#ifndef LOGINSCREEN_H
#define LOGINSCREEN_H

#include <QWidget>
#include <QProcess>
#include <QDateTime>
#include <QPushButton>
#include <QVBoxLayout>
#include <QDialog>
#include <QThread>
#include <QApplication>
#include <QPoint>
#include <QSize>
#include <QDebug>
#include <QSettings>
#include <QFile>
#include <QStandardPaths>
#include <QMessageBox>
#include <QTimer>
#include <QMenu>

#include "settings.h"
#include "session.h"
#include "manager.h"
#include "lwclient.h"
#include "winmanager.h"

#define RDP_TIMEOUT 6 // seconds
#define PROCESS_TIMEOUT 3000 // miliseconds

namespace Ui {
class LoginScreen;
class Widget;
}

class LoginScreen : public QWidget
{
    Q_OBJECT

public:
	explicit LoginScreen(QWidget *parent = 0);
	~LoginScreen();

private slots:
	void shutDown();
	void reboot();
	void suspend();
	void hibernate();
	bool connectToServer();
	void on_loginButton_clicked();
	void finished(int);
	void finished();
	void serversChanged();
	void on_passwordLineEdit_returnPressed();
	void on_loginLineEdit_returnPressed();
	void socketError(QString socketError);
	void launchProcess();
	QString credentialsFile();
	void mountSmbShares();
	bool smbMount(QString credentialsFile, QString smbPath, QString targetPath);
	void smbUmountAll();
	void launchRemovableManagerProcess();
	void on_powerOffButton_clicked();

private:
	void forceTranslations();
	void setShutdownMenu();

	Ui::LoginScreen *ui;

	Manager * manager;
	QString tempFile;
	QDateTime processStart;
	Settings * settings;
	QProcess * removableManagerProcess;
	bool errorShown;
	QString hostName;
	QString server;
	int connPort;
};

#endif // LOGINSCREEN_H
