#ifndef LOGINSCREEN_H
#define LOGINSCREEN_H

#include <QApplication>
#include <QDesktopWidget>
#include <QWidget>
#include <QPixmap>
#include <QPropertyAnimation>
#include <QDir>
#include <QSettings>
#include <QTime>
#include <QDate>
#include <QLabel>
#include <QTimer>
#include <QLineEdit>
#include <QPoint>
#include <QSize>
#include <QComboBox>
#include <QPushButton>
#include <QProcess>
#include <QMenu>
#include <QProxyStyle>
#include <QScreen>

#include "pixmapframe.h"
#include "constants.h"
#include "session.h"
#include "manager.h"
#include "power.h"

#ifdef Q_OS_LINUX
#include <librework-client/clientsession.h>
#include <librework-client/clientsettings.h>
#endif



class MenuStyle : public QProxyStyle
{
public:
	MenuStyle(int iconSize = 24) { smallIconSize = iconSize; };
	int pixelMetric(PixelMetric metric, const QStyleOption * option = 0, const QWidget * widget = 0 ) const
	{
		switch( metric )
		{
		case QStyle::PM_SmallIconSize:
		case QStyle::PM_ButtonIconSize:
		case QStyle::PM_IconViewIconSize:
		case QStyle::PM_ListViewIconSize:
		case QStyle::PM_ToolBarIconSize:
		case QStyle::PM_LargeIconSize:
			return smallIconSize;
		default:
			return QProxyStyle::pixelMetric( metric, option, widget );
		}
	}
private:
	int smallIconSize;
};

class LoginScreen : public QWidget
{
	Q_OBJECT

public:
	LoginScreen( QWidget * parent = 0 );
	~LoginScreen();

	QPixmap selectPixmap( QString configurationFile );

public slots:
	void doAnimation(QKeyEvent *event = 0);
	void doInitScreen();
	void animationFinished();
	void setDateTime();
	void passwordTextChanged(const QString &);

private slots:
	void usernameLineEditTextChanged(QString text);
	void loginButtonClicked();
	void serverChanged(QString server);

private:
	void forceTranslations();
	QString selectConfigurationFile();
	void initInitScreen();
	void initUserControls();
	void showUserControls(bool show);
	void setShutdownMenu();
	QString getMacAddress();

	QTimer * timer;
	QPropertyAnimation * animation;
	Manager * manager;
#ifdef Q_OS_LINUX
	ClientSession * clientSession;
#endif

	PixmapFrame * pixmapFrame;
	QLabel * dateTimeLabel;
	QLabel * logoLabel;
	QLineEdit * usernameLineEdit;
	QLineEdit * passwordLineEdit;
	QComboBox * serverComboBox;
	QPushButton * loginButton;
	QPushButton * exitButton;

	QScreen * screen;
};


#endif
