#ifndef MANAGER_H
#define MANAGER_H

#include <xcb/xcb.h>

#include <QCoreApplication>
#include <QApplication>

#include <QX11Info>
#include <QAbstractEventDispatcher>
#include <QAbstractNativeEventFilter>
#include <QIcon>
#include <QString>

#include "constants.h"
#include "decorator.h"
#include "settings.h"
#include "desktop.h"

class Desktop;

#include <X11/X.h>

class Manager : public QObject, QAbstractNativeEventFilter
{
	Q_OBJECT

	public:
		Manager(Desktop * parent);
		~Manager();

		static bool eventFilter(void * message);
		void showWindow(Window window);

	signals:
		void fileOpened();
		void titleChanged( long long, QString );
		void mapWindow( long long );
		void mapNotify( long long );
		void unmapWindow( long long );

	private:
		bool nativeEventFilter(const QByteArray & eventType, void * message, long * result);
		void attachWindow(QWidget * container, Window window);
		void resizeWindow( Window window, bool isDialog = false, QSize preferredSize = QSize(0, 0) );
		void decorateWindow(Window window, QSize size, QPoint pos);

		bool buttonPressEvent( xcb_generic_event_t * event );
		bool propertyNotifyEvent( xcb_generic_event_t * event );
		bool mapRequestEvent( xcb_generic_event_t * event );
		bool configureRequestEvent( xcb_generic_event_t * event );
		bool destroyNotifyEvent( xcb_generic_event_t * event );
		bool unmapNotifyEvent( xcb_generic_event_t * event );
		bool createNotifyEvent( xcb_generic_event_t * event );
		bool mapNotifyEvent( xcb_generic_event_t * event );

		void mapSplash(Window window);
		void mapDialog(Window window);

		Display * dpy;
		QList<Q_PID> pids;
		LWClient * lwClient;

		QIcon currentIcon;
		QString currentName;
		Desktop * desktop;
};

static QList<Manager*> instance;

#endif
