#ifndef SESSION_H
#define SESSION_H

#include <QByteArray>
#include <QString>
#include <QFile>
#include <QDir>
#include <QProcess>
#include <QWidget>
#include <QStandardPaths>
#include <QObject>
#include <QTranslator>

#ifdef Q_OS_LINUX
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <grp.h>
#include <pwd.h>
#include <security/pam_appl.h>
#endif

#ifdef Q_OS_WIN
#include <WinSock2.h>
#include <Windows.h>
#endif

#include "settings.h"
#include "initprogress.h"

class Manager;

class Session : public QObject
{
	public:
		static void getUserAndGroupIds( QString username, int * uid, int * gid );
		static QString getUserHome( QString username );
		static int setCurrentDir(const char *dirname);
		static QString createHome( QString homes, QString username );
		static bool switchUser(QString homes, QString username );
		static void createDesktopLinks(QString homeDir, QString username, bool setPermissions = false);
		static bool createSystemUser(int uid, QString homes, QString username );
		static bool authenticateSystemUser( QString username, QString password );
		static bool openSession();
		static bool closeSession();
		bool pamValidate( const char * username, const char * password);
		static QStringList launch(QString username, QString password, bool localSession, QWidget *parent, Manager * &manager, InitProgress *initProgress);
		static bool connectToServer();
		static QString credentialsFile(QString username, QString password);
		static void mountSmbShares(QString username, QString password);
		static bool smbMount(QString username, QString credentialsFile, QString smbPath, QString targetPath);
		static void smbUmountAll();
};

#endif //SESSION_H
