#include "wininfo.h"

#ifdef Q_OS_LINUX
#include "x11info.h"
#endif

void WinInfo::selectWindow(HWND hWnd, bool fullScreen)
{
#ifdef Q_OS_WIN
	if ( IsIconic( hWnd ) )
	{
		ShowWindow( hWnd, SW_RESTORE );
	} else {
		ShowWindow( hWnd, SW_SHOW );
	}
    BringWindowToTop( hWnd );
    if ( Settings::getWindowManagerBehaviour() == FULLSCREEN || fullScreen )
    {
        ShowWindow(hWnd, SW_SHOWMAXIMIZED);
    }
#endif
#ifdef Q_OS_LINUX
	X11Info::showWindow( (Window) hWnd );
#endif
}

void WinInfo::followProc(DWORD processId)
{
    qDebug("Following process %d", (int)processId);
    //SetWinEventHook(EVENT_SYSTEM_END, EVENT_SYSTEM_END, NULL, FollowWindowsProc, NULL /*processId*/, NULL, WINEVENT_OUTOFCONTEXT | WINEVENT_SKIPOWNTHREAD);
}

/*
void CALLBACK FollowWindowsProc(HWINEVENTHOOK hook, DWORD event, HWND hWnd, long idObject, long idChild, DWORD dwEventThread, DWORD dwmsEventTime)
{
    qDebug("Followed event %d on window %d", (int) event, (int) hWnd);
}
*/

/*QList<HWND> WinInfo::getWindows(QProcess * process)
{
    handleData data;
    data.process = process;
    data.windows.clear();
    data.first = (HWND) -1;
    EnumWindows(EnumWindowsProc, (LPARAM)&data);
    return data.windows;
}*/

HWND WinInfo::getWindow(QProcess * process)
{
#ifdef Q_OS_WIN
	handleData data;
	data.process = process;
	data.windows.clear();
	data.first = (HWND) -1;
	EnumWindows(FindWindowProc, (LPARAM)&data);
	return data.first;
#endif
#ifdef Q_OS_LINUX
	QList<Window> list = X11Info::getProcessWindows( process->pid() );
	if ( list.count() > 0 )
	{
		return list[0];
	} else {
		return -1;
	}
#endif
}

QList<HWND> WinInfo::getVisibleWindows()
{
#ifdef Q_OS_WIN
    handleData data;
    data.process = NULL; // enum all visible windows
    data.windows.clear();
    //data.first = (HWND) -1;
    EnumWindows(EnumWindowsProc, (LPARAM)&data);
    EnumChildWindows(WinInfo::fromId(QApplication::desktop()->winId()), EnumWindowsProc, (LPARAM)&data );
    return data.windows;
#endif
#ifdef Q_OS_LINUX
	QList<HWND> ret = (QList<HWND>) X11Info::getVisibleWindows();
	return ret;
#endif
}

void WinInfo::debugWindowInfo(HWND hWnd)
{
#ifdef Q_OS_WIN
    qDebug("Detected window: %d '%s' with parent: %d isToolWindow: %d isAppWindow: %d isDlgFrame: %d isChild: %d isPopup: %d isPopupWindow: %d  hasSysmenu: %d isOverlapped: %d, isOverlappedWindow: %d, isVisible: %d",
		(int)hWnd,
        WinInfo::getWindowTitle(hWnd).toUtf8().data(),
		(int)WinInfo::getParent(hWnd),
        (int)WinInfo::isToolWindow(hWnd),
        (int)WinInfo::isAppWindow(hWnd),
        (int)WinInfo::isDlgFrame(hWnd),
        (int)WinInfo::isChild(hWnd),
        (int)WinInfo::isPopup(hWnd),
        (int)WinInfo::isPopupWindow(hWnd),
        (int)WinInfo::hasSysmenu(hWnd),
        (int)WinInfo::isOverlapped(hWnd),
        (int)WinInfo::isOverlappedWindow(hWnd),
        (int)WinInfo::isVisible(hWnd)
    );
#endif
}

void WinInfo::setWindowsInfo(QList<HWND> * windowsList, QList<long> * styles, QList<long> * exStyles, QList<QString> * titles, QList<unsigned long> *pids)
{
	Q_UNUSED(pids);
    windowsList->clear();
    styles->clear();
    exStyles->clear();
    titles->clear();
    QList<HWND> list = WinInfo::getVisibleWindows();
    for (int i = 0; i < list.count(); i++)
    {
		/*if (WinInfo::getWindowExecName( list[i] ) == "explorer.exe")
		{
			qDebug("Eureka!");
		} else {
			QString name = WinInfo::getWindowExecName( list[i] );
			//if (! name.isEmpty())
				//qDebug("Finding explorer... Exec name: %s Window pid: %d", name.toUtf8().data(), WinInfo::getWindowPid(list[i]));
		}*/
        if ( ! windowsList->contains(list[i]) ) // Don't add duplicates!
        {
#ifdef Q_OS_WIN
            tagWINDOWINFO pwi;
            GetWindowInfo(list[i], &pwi);
            styles->append(  pwi.dwStyle );
            exStyles->append( pwi.dwExStyle );
#endif
			QString title = WinInfo::getWindowTitle( list[i] );
			windowsList->append( list[i] );
            titles->append(title);
			//pids->append(getWindowPid(list[i]));
        }
    }
}

unsigned long WinInfo::getWindowPid(HWND hWnd)
{
    unsigned long processId = 0;
#ifdef Q_OS_WIN
    GetWindowThreadProcessId(hWnd, &processId);
#endif
#ifdef Q_OS_LINUX
	processId = X11Info::getWindowPid( (Window) hWnd );
#endif
    return processId;
}

#ifdef Q_OS_WIN
BOOL CALLBACK FindWindowProc(HWND hWnd, LPARAM lparam)
{
	handleData * data = (handleData*)lparam;
	DWORD lpdwProcessId;
	GetWindowThreadProcessId(hWnd, &lpdwProcessId);
	if( lpdwProcessId == data->process->processId() )
	{
		data->first = hWnd;
		return FALSE;
	}
	return TRUE;
}

BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lparam)
{
    handleData * data = (handleData*)lparam;
    //qDebug("data->first: %d, hWnd: %d, Title: %s", (int)data->first, (int)hWnd, WinInfo::getWindowTitle(hWnd).toUtf8().data());
    /*if (data->first == (HWND) -1)
    {
        data->first = hWnd;
    } else {
        if (data->first == hWnd)
        {
            qDebug("Returning false!!! Count: %d", data->windows.count());
            return false;
        }
    }*/

    if ( data->process != NULL ) // enum windows for that process
    {
        unsigned long processId = 0;
        GetWindowThreadProcessId(hWnd, &processId);
        if (data->process->processId() == processId)
        {
            qDebug("Process found!!! %d", (int)processId);
            data->windows += hWnd;
        }
        return true;
    } else { // enum all  windows
        if ( WinInfo::isVisible(hWnd) )
        {
            data->windows += hWnd;
        }
        return true;
    }
}
#endif

QString WinInfo::getWindowTitle(HWND hWnd)
{
#ifdef Q_OS_WIN
    int maxString = 256;
    LPWSTR title = new WCHAR[maxString + 1];
    int result = GetWindowText(hWnd, title, maxString);
    QString titleString = QString::fromWCharArray(title);
    delete title;
    if (result > 0)
    {
        return QString(titleString);
    } else {
        return QString::null;
    }
#endif
#ifdef Q_OS_LINUX
	return X11Info::getWindowName( (Window) hWnd );
#endif
}

QString WinInfo::getWindowExecName(HWND hWnd)
{
    unsigned long pid = getWindowPid(hWnd);
#ifdef Q_OS_WIN
    LPWSTR execName = new WCHAR[256];
    HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS, TRUE,(DWORD) pid);
    if ( hProcess == NULL)
    {
        delete execName;
        return QString::null;
    }
    int result = GetModuleFileNameEx(hProcess, NULL, execName, 256);
    QString execNameString = QString::fromWCharArray(execName);
    delete execName;
    if (result > 0)
    {
		return QString(execNameString).section("\\", -1, -1);
    } else {
        return QString::null;
    }
#endif
#ifdef Q_OS_LINUX
	QString ret;
	QString fileName = "/proc/" + QString::number( pid ) + "/cmdline";
	if ( QFile::exists( fileName ) )
	{
		QFile file(fileName);
		if( file.open(QIODevice::ReadOnly) )
		{
			QTextStream in(&file);
			while(!in.atEnd())
			{
				QString line = in.readLine();
				ret += line;
			}
			file.close();
		}
	}
	return ret;
#endif
}

HWND WinInfo::getParent(HWND hWnd)
{
#ifdef Q_OS_WIN
	return GetParent( hWnd );
#endif
#ifdef Q_OS_LINUX
	return X11Info::getParent( (Window) hWnd );
#endif
}

bool WinInfo::hasNoParent(HWND hWnd)
{
#ifdef Q_OS_WIN
	HWND hParent = GetAncestor(hWnd, GA_PARENT);
	return  ! hParent || hParent == GetDesktopWindow();
#endif
#ifdef Q_OS_LINUX
	return getParent( hWnd ) == 0;
#endif
}

#ifdef Q_OS_WIN
int WinInfo::isWindowStyle(HWND hWnd, long style)
{
    /*tagWINDOWINFO pwi;
    GetWindowInfo(hWnd, &pwi);
    return pwi.dwStyle & style;*/
    long gwlStyle = GetWindowLong(hWnd, GWL_STYLE);
    return gwlStyle & style;
}

int WinInfo::isWindowExStyle(HWND hWnd, long exStyle)
{
    long gwlExStyle = GetWindowLong(hWnd, GWL_EXSTYLE);
    //qDebug("gwlExStyle: %#06x exStyle: %#06x isExStyle: %d", gwlExStyle, exStyle, gwlExStyle & exStyle );
    return gwlExStyle & exStyle;
}

bool WinInfo::isChild(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_CHILD );
}

bool WinInfo::isDlgFrame(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_DLGFRAME );
}

bool WinInfo::isPopupWindow(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_POPUPWINDOW);
}

bool WinInfo::isPopup(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_POPUP);
}

bool WinInfo::hasSysmenu(HWND hWnd)
{
    return isWindowStyle(hWnd, WS_SYSMENU);
}

bool WinInfo::isOverlapped(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_OVERLAPPED );
}

bool WinInfo::isOverlappedWindow(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_OVERLAPPEDWINDOW);
}
bool WinInfo::isVisible(HWND hWnd)
{
    return isWindowStyle( hWnd, WS_VISIBLE);
}

bool WinInfo::isAppWindow(HWND hWnd)
{
    return isWindowExStyle( hWnd, WS_EX_APPWINDOW);
}

bool WinInfo::isToolWindow(HWND hWnd)
{
    return isWindowExStyle( hWnd, WS_EX_TOOLWINDOW);
}
#endif
void WinInfo::closeWinWindow(HWND hWnd)
{
#ifdef Q_OS_WIN
    PostMessage(hWnd, WM_SYSCOMMAND, SC_CLOSE, 0);
#endif
#ifdef Q_OS_LINUX
	X11Info::closeXorgWindow( (Window) hWnd );
#endif
}

void WinInfo::killWinWindow(HWND hWnd)
{
#ifdef Q_OS_WIN
    PostMessage(hWnd, WM_QUIT, 0, 0);
#endif
#ifdef Q_OS_LINUX
	X11Info::killXorgWindow( (Window) hWnd );
#endif
}

HWND WinInfo::fromId(long long id)
{
    return (HWND) id;
    /*
    QWindow * window = QWindow::fromWinId((WId)id);
    HWND ww = (HWND) QGuiApplication::platformNativeInterface()->nativeResourceForWindow(QByteArrayLiteral("handle"), window);
    return ww;
    */
}

void WinInfo::attachWindow(QWidget * container, HWND hWnd)
{
#ifdef Q_OS_WIN
    DWORD dwStyle = GetWindowLong( hWnd, GWL_STYLE );
    dwStyle &= ~WS_BORDER;
    dwStyle &= ~WS_THICKFRAME;
    DWORD dwExStyle = GetWindowLong( hWnd, GWL_EXSTYLE );
    dwExStyle |= WS_EX_APPWINDOW;
    QWindow * window = QWindow::fromWinId((long long)hWnd);
    Q_UNUSED( window );
    //SetParent(hWnd, fromId(container->windowHandle()->winId()));
    SetWindowLong( hWnd, GWL_STYLE, dwStyle);
    SetWindowLong( hWnd, GWL_EXSTYLE, dwExStyle);
    MoveWindow( hWnd, 0, 0, container->width(), container->height(), TRUE);
    WinInfo::selectWindow( hWnd );
#endif
#ifdef Q_OS_LINUX
	return X11Info::attachWindow( container, (Window) hWnd );
#endif
}

void WinInfo::hideTaskBar()
{
#ifdef Q_OS_WIN
    APPBARDATA barData;
    barData.cbSize = sizeof( barData );
    barData.hWnd = FindWindow( L"System_TrayWnd", NULL);
    barData.lParam = ABS_AUTOHIDE;
    SHAppBarMessage( ABM_SETSTATE, &barData );
#endif
}

void WinInfo::showTaskBar()
{
#ifdef Q_OS_WIN
    APPBARDATA barData;
    barData.cbSize = sizeof( barData );
    barData.hWnd = FindWindow( L"System_TrayWnd", NULL);
    barData.lParam = ABS_ALWAYSONTOP;
    SHAppBarMessage( ABM_SETSTATE, &barData );
#endif
}

#ifdef Q_OS_WIN
void WinInfo::setWallPaper(LPWSTR image)
{
    int res = SystemParametersInfo( SPI_SETDESKWALLPAPER, 0, image, SPIF_UPDATEINIFILE);
    Q_UNUSED(res);
}
#endif

QIcon WinInfo::getWindowIcon(HWND hwnd)
{
#ifdef Q_OS_WIN
    HICON iconHandle = (HICON)SendMessage(hwnd,WM_GETICON,ICON_SMALL2,0);
    if(iconHandle == 0)
        iconHandle = (HICON)SendMessage(hwnd,WM_GETICON,ICON_SMALL,0);
    if(iconHandle == 0)
        iconHandle = (HICON)SendMessage(hwnd,WM_GETICON,ICON_BIG,0);
/*    if (iconHandle == 0)
        iconHandle = (HICON)GetClassLongPtr(hwnd, GCL_HICON);
    if (iconHandle == 0)
        iconHandle = (HICON)GetClassLongPtr(hwnd, GCL_HICONSM);
*/
    if(iconHandle == 0)
        return QIcon();
    //Icon icn = Icon.FromHandle(iconHandle);

    QIcon icon = QtWin::fromHICON(iconHandle);

	return icon;
#endif
#ifdef Q_OS_LINUX
	return X11Info::getWindowIcon((Window) hwnd);
#endif
}

QIcon WinInfo::getAppIcon(QString command)
{
    QIcon icon;
#ifdef Q_OS_WIN
    LPCWSTR wCommand = (LPCWSTR)command.utf16();
    HICON hIcon = ExtractIcon(NULL, (LPCWSTR)wCommand, 0);
    if (hIcon != NULL)
    {
        icon = QtWin::fromHICON( hIcon );
    }
#endif
    return icon;
}
