#include "x11info.h"

#ifdef Q_OS_LINUX

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/extensions/XShm.h>

X11Info::X11Info()
{
}

QList<Window> X11Info::getProcessWindows(Q_PID pid)
{
	Window wRoot = XDefaultRootWindow(QX11Info::display());

	QList<Window> result;
	QList<Window> wChild;

	wChild = getAllChildren( wRoot );

	for (int i=0; i < wChild.count(); i++)
	{
		Q_PID wPid = getWindowPid(wChild[i]);
		if (getWindowName(wChild[i]).startsWith("SeamlessRDP") || getWindowName(wChild[i]).startsWith("FreeRDP") || getWindowName(wChild[i]).startsWith("rdesktop") || wPid == pid)
		{
			result += wChild[i];
		} else {
			//qDebug("Not attaching Window '%s' id: %x, as pid is %d, not %d", getWindowName(wChild[i]).toUtf8().data(), (int)wChild[i], wPid, pid);
		}
	}
	return result;
}

QList<Window> X11Info::getChildren(Window window)
{
	unsigned int count;
	Window rootWindow;
	QList<Window> res;

	Window wParent;
	Window * wChild = new Window;

	if( ! XQueryTree(QX11Info::display(), window, &rootWindow, &wParent, &wChild, &count) != 0)
		return res;

	for (uint i = 0; i < count; i++)
	{
		res += wChild[i];
	}

	XFree(wChild);

	return res;
}

Window X11Info::getParent(Window window)
{
	unsigned int count;
	Window rootWindow;

	Window wParent;
	Window * wChild = new Window;

	if( XQueryTree(QX11Info::display(), window, &rootWindow, &wParent, &wChild, &count) == 0)
		return 0;

	XFree(wChild);

	return wParent;
}

QList<Window> X11Info::getAllChildren(Window window)
{
	QList<Window> list;

	Window rootWindow;
	Window wParent;
	Window * wChild = new Window;
	unsigned int count;
	
	if( XQueryTree(QX11Info::display(), window, &rootWindow, &wParent, &wChild, &count) != 0)
	{
		for (uint i=0; i<count; i++)
		{
			list += wChild[i];
			list += getAllChildren(wChild[i]);
		}
		XFree(wChild);
	}
	return list;
}

QPair<QString,QString> X11Info::getWindowClass(Window window)
{
	XClassHint classHints;
	QString _name;
	QString _class;
	if ( XGetClassHint(QX11Info::display(), window, &classHints) != 0 )
	{
		_name = classHints.res_name;
		_class = classHints.res_class;
		XFree( classHints.res_name );
		XFree( classHints.res_class );
	}
	QPair<QString,QString> ret;
	ret.first = _name;
	ret.second = _class;
	return ret;
}

long X11Info::getRailWindow( Window window )
{
	QPair<QString,QString> windowClass = X11Info::getWindowClass( window );
	qDebug("Window id: %d, class name: %s, class: %s", (int)window, windowClass.first.toUtf8().data(), windowClass.second.toUtf8().data());

	long railWindow = 0;

	if ( windowClass.first == "RAIL" )
	{
		QString windowString = windowClass.second.section(":", 1, 1);
		if ( ! windowString.isEmpty() )
		{
			bool ok;
			railWindow = windowString.toLong( &ok, 16 );
			if ( ! ok )
			{
				railWindow = 0;
			}
		}
	}

	return railWindow;
}

void X11Info::setWorkArea(QRect rect)
{
	/*
	unsigned long data[4];
	data[0] = rect.left();
	data[1] = rect.top();
	data[2] = rect.right();
	data[3] = rect.bottom();
	Atom a = XInternAtom(QX11Info::display(), "_NET_WORKAREA", true);
	int status = XChangeProperty(QX11Info::display(), XDefaultRootWindow(QX11Info::display()), a, XA_CARDINAL, 32, PropModeReplace, (unsigned char*)data, 4);
	*/

	unsigned long data[12];
	data[0] = rect.left(); // left
	data[1] = rect.right(); // right
	data[2] = rect.top(); // top
	data[3] = rect.bottom(); // bottom
	data[4] = rect.bottom(); //left_start_y
	data[5] = QGuiApplication::screens()[0]->geometry().height(); // left_end_y
	data[6] = rect.bottom(); //right_start_y
	data[7] = QGuiApplication::screens()[0]->geometry().height(); // right_end_y
	data[8] = rect.left(); // top_start_x
	data[9] = rect.right(); // top_end_x
	data[10] = rect.left(); // bottom_start_x
	data[9] = rect.right(); // bottom_end_x
	Atom a = XInternAtom(QX11Info::display(), "_NET_WM_STRUT_PARTIAL", true);
	int status = XChangeProperty(QX11Info::display(), XDefaultRootWindow(QX11Info::display()), a, XA_CARDINAL, 32, PropModeReplace, (unsigned char*)data, 4);
	if (status >= Success)
	{
		qDebug("Changed default work area to %d, %d, %d, %d", (int)data[0], (int)data[1], (int)data[2], (int)data[3]);
	} else {
		qDebug("something failed setting work area. Error: %d", status);
	}
}

void X11Info::setDesktopGeometry(int w, int h)
{
	unsigned long data[2];
	data[0] = w;
	data[1] = h;
	Atom a = XInternAtom(QX11Info::display(), "_NET_DESKTOP_GEOMETRY", true);
	int status = XChangeProperty(QX11Info::display(), XDefaultRootWindow(QX11Info::display()), a, XA_CARDINAL, 32, PropModeReplace, (unsigned char*)data, 2);
	if (status >= Success)
	{
		qDebug("Changed default desktop geometry to %d, %d", (int)data[0], (int)data[1]);
	} else {
		qDebug("something failed setting desktop geometry. Error: %d", status);
	}
}

void X11Info::setWindowPosition(Window window, int x, int y)
{
	Window ret;
	XMoveWindow( QX11Info::display(), window, x, y);
	//XTranslateCoordinates( QX11Info::display(), window, XDefaultRootWindow(QX11Info::display()), 0, 0, &x, &y, &ret );
}

void X11Info::setWindowDock(Window window)
{
	Atom a = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", True);

	if (a != None)
	{
		Atom prop = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE_DOCK", True);
		XChangeProperty(QX11Info::display(), window, a, XA_ATOM, 32, PropModeReplace, (unsigned char *) &prop, 1);
	}
}

void X11Info::setDesktopViewport(int x, int y)
{
	unsigned long data[2];
	data[0] = x;
	data[1] = y;
	Atom a = XInternAtom(QX11Info::display(), "_NET_DESKTOP_VIEWPORT", true);
	int status = XChangeProperty(QX11Info::display(), XDefaultRootWindow(QX11Info::display()), a, XA_CARDINAL, 32, PropModeReplace, (unsigned char*)data, 2);
	if (status >= Success)
	{
		qDebug("Changed default viewport corner to %d, %d", (int)data[0], (int)data[1]);
	} else {
		qDebug("something failed setting desktop viewport corner. Error: %d", status);
	}
}

QList<long long> X11Info::getVisibleWindows()
{
	QList<long long> ret;
	Atom a = XInternAtom(QX11Info::display(), "_NET_CLIENT_LIST" , true);
	Atom actualType;
	int format;
	unsigned long numItems, bytesAfter;
	unsigned char *data = 0;

	int status = XGetWindowProperty(QX11Info::display(),
									XDefaultRootWindow(QX11Info::display()),
									a,
									0L,
									(~0L),
									false,
									AnyPropertyType,
									&actualType,
									&format,
									&numItems,
									&bytesAfter,
									&data);

	if (status >= Success && numItems)
	{
		long * array = (long*) data;
		for (unsigned int k = 0; k < numItems; k++)
		{
			long long window = (long long) array[k];
			ret.append( window );
		}
		XFree(data);
	}
	return ret;
}

void X11Info::moveToBack(Window window)
{
	XLowerWindow(QX11Info::display(), window);
}

Q_PID X11Info::getWindowPid(Window window)
{
	Atom type;
	int format;
	unsigned long nItems;
	unsigned long bytesAfter;
	unsigned char *propPID = 0;
	Q_PID pid = 0;

	Atom _atomPID = XInternAtom(QX11Info::display(), "_NET_WM_PID", True);

	int result = XGetWindowProperty(QX11Info::display(), window, _atomPID, 0, 1, False, XA_CARDINAL,
					&type, &format, &nItems, &bytesAfter, &propPID);
	if (result == Success)
	{
		if(propPID != 0)
		{
			pid = *((Q_PID *)propPID);
			XFree(propPID);
		}
	}

	return pid;
}

QString X11Info::getWindowTextProperty(Window window, QString property)
{
	XTextProperty tp;
	Atom atom = XInternAtom(QX11Info::display(), property.toUtf8().data(), False);
	Atom utf8 = XInternAtom(QX11Info::display(), "UTF8_STRING", False);
	char ** data = NULL;
	QString result = QString::null;

	if ( XGetTextProperty( QX11Info::display(), window, &tp, atom ) && tp.value != NULL )
	{
		if ( tp.encoding == utf8 )
		{
			result = QString::fromUtf8( (const char *)tp.value );
		} else if ( tp.encoding == XA_STRING ) {
			result = QString::fromUtf8( (const char*)tp.value );
		} else {
			result = QString( (const char *)tp.value );
		}
		if( data != NULL )
		{
			XFreeStringList( data );
		}
		XFree( tp.value );
	}

	return result;
}

QString X11Info::getWindowName(Window window)
{
	QString res = getWindowTextProperty(window, "_NET_WM_NAME");

	if (res.isEmpty())
		res = getWindowTextProperty(window, "XA_WM_NAME");

	if (res.isEmpty())
		res = getWindowTextProperty(window, "WM_NAME");

	return res;
}

QIcon X11Info::getWindowIcon(Window window)
{
	unsigned long nitems, bytesafter;
	unsigned char *data;
	int format;
	Atom type;

	Atom net_wm_icon = XInternAtom(QX11Info::display(), "_NET_WM_ICON", False);

	/* Get icon width */

	XGetWindowProperty(QX11Info::display(), window, net_wm_icon, 0, 1, False, XA_CARDINAL, &type, &format, &nitems, &bytesafter, &data);

	if (!data)
	{
		// Failed to read width
		return QPixmap();
	}

	int width = data[0];

	qDebug("Icon width: %d", width);
	XFree(data);
  
	/* Get icon height */

	XGetWindowProperty(QX11Info::display(), window, net_wm_icon, 1, 1, False, XA_CARDINAL, &type, &format, &nitems, &bytesafter, &data);

	if (!data)
	{
		// Failed to read height
		return QPixmap();
	}

	int height = data[0];
	qDebug("Icon height: %d", height);
	XFree(data);

	/* Get icon data. */

	XGetWindowProperty(QX11Info::display(), window, net_wm_icon, 2, width * height, False, XA_CARDINAL, &type, &format, &nitems, &bytesafter, (unsigned char **)&data);

	if (!data)
	{
		// Failed to read data
		return QPixmap();
	}

	QImage img(data, width, height, QImage::Format_ARGB32);
        QPixmap pixmap = QPixmap::fromImage(img);

	XFree(data);

	return QIcon(pixmap);
}

bool X11Info::isValidWindow( Window window )
{
	XWindowAttributes attributes;

	Status ret = XGetWindowAttributes(QX11Info::display(), window, &attributes);

	if ( ret == 0 )
	{
		return false;
	} else {
		return true;
	}
}

int X11Info::getWindowMapState(Window window)
{
	XWindowAttributes attributes;

	XGetWindowAttributes(QX11Info::display(), window, &attributes);

	int result = attributes.map_state;

	return result;
}

bool X11Info::getWindowSaveUnder(Window window)
{
	XWindowAttributes attributes;

	XGetWindowAttributes(QX11Info::display(), window, &attributes);

	bool result = attributes.save_under;

	return result;
}

QSize X11Info::getWindowMinSize(Window window)
{
	long msize;
	XSizeHints sizeHint;
	XGetWMNormalHints(QX11Info::display(), window, &sizeHint, &msize);
	return QSize(
			(sizeHint.flags & PMinSize) ? sizeHint.min_width : 0,
			(sizeHint.flags & PMinSize) ? sizeHint.min_height : 0
	);
}

QSize X11Info::getWindowBaseSize(Window window)
{
	long bsize;
	XSizeHints sizeHint;
	XGetWMNormalHints(QX11Info::display(), window, &sizeHint, &bsize);
	QSize ret = QSize(
			(sizeHint.flags & PBaseSize) ? sizeHint.base_width : 0,
			(sizeHint.flags & PBaseSize) ? sizeHint.base_height : 0
	);

	if ( ret == QSize( 0, 0 ) )
		ret = getWindowMinSize( window );

	return ret;
}

QSize X11Info::getWindowMaxSize(Window window)
{
	long msize;
	XSizeHints sizeHint;
	XGetWMNormalHints(QX11Info::display(), window, &sizeHint, &msize);
	QSize ret = QSize(
			(sizeHint.flags & PMaxSize) ? sizeHint.max_width : 0,
			(sizeHint.flags & PMaxSize) ? sizeHint.max_height : 0
	);

	if ( ret == QSize( 0, 0 ) )
		ret = getWindowBaseSize( window );

	return ret;
}

QSize X11Info::getWindowPreferredSize(Window window)
{

        Window root_return;
	int x_return, y_return;
	unsigned int width_return, height_return;
	unsigned int border_width_return;
	unsigned int depth_return;

	XGetGeometry(QX11Info::display(), window, &root_return, &x_return, &y_return, &width_return, 
                      &height_return, &border_width_return, &depth_return);

	QSize ret = QSize( width_return, height_return );

	if ( ret == QSize( 0, 0 ) )
		ret = getWindowMinSize( window );

	return ret;
}

QString X11Info::getWindowType(Window window)
{

	Atom actType;
	int actFormat;
	unsigned long nItems, bytes;
	Atom windowType;

	Atom netWmName = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", False);

	char * data;

	int result = XGetWindowProperty(QX11Info::display(), window, netWmName, 0, 1, False, XA_ATOM, &actType, &actFormat, &nItems, &bytes, (unsigned char **) &data);

	if (result == Success)
	{
		if (data != 0)
		{
			windowType = *(Atom *) data;
			XFree (data);
		}
	}

	Display * xdisplay = QX11Info::display();

	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DESKTOP", False))
		return QString("Desktop");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DOCK", False))
		return QString("Dock");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_TOOLBAR", False))
		return QString("Toolbar");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_MENU", False))
		return QString("Menu");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_UTILITY", False))
		return QString("Utility");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_SPLASH", False))
		return QString("Splash");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DIALOG", False))
		return QString("Dialog");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU", False))
		return QString("Dropdown Menu");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_POPUP_MENU", False))
		return QString("Popup Menu");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_TOOLTIP", False))
		return QString("Tooltip");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_NOTIFICATION", False))
		return QString("Notification");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_COMBO", False))
		return QString("Combo");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DND", False))
		return QString("DND");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_NORMAL", False))
		return QString("Normal");

	return QString("Unknown %1").arg((uint)windowType);
}

void X11Info::showWindowsTree(Window window, int depth)
{
	showWindowInfo(window, depth);

	QList<Window> rootWindows = X11Info::getChildren(window);

	int count = rootWindows.count();

	for (int i = 0; i < count; i++)
		showWindowsTree(rootWindows[i], depth + 1);
}

void X11Info::showWindowInfo(Window window, int depth)
{
	int tabSize = 4;

	QString spaces = QString().fill(' ', tabSize * depth);

	Q_PID pid = X11Info::getWindowPid(window);

	QString name = X11Info::getWindowName(window);
	QString wtype = X11Info::getWindowType(window);

	QString command = "--no command--";

	QFile proc(QString("/proc/%1/cmdline").arg(pid));
	if (proc.open(QIODevice::ReadOnly))
	{
		command = proc.readAll();
		proc.close();
	}

	if (X11Info::getWindowMapState(window) == IsViewable)
		qDebug("%sWindow id: %d; Window type: %s, name: %s, pid: %d, command line: %s",
					spaces.toUtf8().data(),
					(int)window,
					wtype.toUtf8().data(),
					name.toUtf8().data(),
					(int)pid,
					command.toUtf8().data());
}

void X11Info::paintRectangle(Window window, QRect rect, char * color)
{
	GC gc;
	XColor xcol;
	Colormap colormap;
	
	colormap = DefaultColormap(QX11Info::display(), 0);

	gc = XCreateGC(QX11Info::display(), window, 0, 0);

	XParseColor(QX11Info::display(), colormap, color, &xcol);
	XAllocColor(QX11Info::display(), colormap, &xcol);

	XSetForeground(QX11Info::display(), gc, xcol.pixel);

	XFillRectangle(QX11Info::display(), window, gc, rect.x(), rect.y(), rect.width(), rect.height() );
}

void X11Info::setInputFocus( Window window )
{
	XSetInputFocus(QX11Info::display(), window, RevertToParent, CurrentTime);
}

void X11Info::sendKeyPressEvent( Window window, int modifiers, int keysym )
{
	if (window != 0)
	{
		XKeyEvent event;

		event.display     = QX11Info::display();
		event.window      = window;
		event.root        = XDefaultRootWindow(QX11Info::display());
		event.subwindow   = None;
		event.time        = CurrentTime;
		event.x           = 1;
		event.y           = 1;
		event.x_root      = 1;
		event.y_root      = 1;
		event.same_screen = True;
		event.keycode     = XKeysymToKeycode(QX11Info::display(), keysym);
		event.state       = modifiers;
		event.type        = KeyPress;
		
		XSendEvent(QX11Info::display(), window, true, KeyPressMask, (XEvent *)&event);
	}
}

void X11Info::disconnectRdpWindow( Window window )
{
	if ( RDP_CLIENT == QString( "rdesktop" ) )
	{
		sendKeyPressEvent( window, ShiftMask, XK_F10 );
	}

	if ( RDP_CLIENT == QString( "xfreerdp" ) )
	{
		sendKeyPressEvent( window, 0, XK_Shift_L );
		sendKeyPressEvent( window, 0, XK_F10 );
	}
}

void X11Info::closeXorgWindow( Window window )
{
	if (window != 0)
	{
		XEvent event;
		event.xclient.type         = ClientMessage;
		event.xclient.window       = window;
		event.xclient.message_type = XInternAtom(QX11Info::display(), "WM_PROTOCOLS", false);
		event.xclient.format       = 32;
		event.xclient.data.l[0]    = XInternAtom(QX11Info::display(), "WM_DELETE_WINDOW", false);
		event.xclient.data.l[1]    = CurrentTime;
		XSendEvent(QX11Info::display(), window, false, StructureNotifyMask, &event);
		XFlush(QX11Info::display());
	}
}

void X11Info::closeWinWindow( Window window )
{
	if ( RDP_CLIENT == QString( "rdesktop" ) )
	{
		sendKeyPressEvent( window, ShiftMask, XK_F11 );
	}

	if ( RDP_CLIENT == QString( "xfreerdp" ) )
	{
		sendKeyPressEvent( window, 0, XK_Shift_L );
		sendKeyPressEvent( window, 0, XK_F11 );
	}
}

void X11Info::killXorgWindow( Window window )
{
	XKillClient(QX11Info::display(), window);
	closeXorgWindow( window );
//	XDestroyWindow(QX11Info::display(), window);
}

void X11Info::killWinWindow( Window window )
{
	if ( RDP_CLIENT == QString( "rdesktop" ) )
	{
		sendKeyPressEvent( window, ShiftMask, XK_F12 );
	}

	if ( RDP_CLIENT == QString( "xfreerdp" ) )
	{
		sendKeyPressEvent( window, 0, XK_Shift_L );
		sendKeyPressEvent( window, 0, XK_F12 );
	}
}

void X11Info::attachWindow(QWidget * container, Window window)
{
        if ( window > 0 && container != 0 )
        {
		qDebug("Attaching window %d", (int)window);
                XWindowAttributes window_attributes_return;
                XGetWindowAttributes(QX11Info::display(), window, &window_attributes_return);
                qDebug("map_state: %d, depth: %d", window_attributes_return.map_state, window_attributes_return.depth);

                XUnmapWindow( QX11Info::display(), window );
                XReparentWindow(QX11Info::display(), window, container->winId(), 0, 0);
				//XMoveResizeWindow(QX11Info::display(), window, 0, 0, container->width(), container->height() - 8);

                XMapRaised(QX11Info::display(), window );

                X11Info::setInputFocus( window );

                XSync(QX11Info::display(), false);
        }
}

void X11Info::attachWindow(Window window)
{
	if ( window > 0 )
	{
		qDebug("Attaching window %d", (int)window);
		XWindowAttributes window_attributes_return;
		XGetWindowAttributes(QX11Info::display(), window, &window_attributes_return);
		qDebug("map_state: %d, depth: %d", window_attributes_return.map_state, window_attributes_return.depth);

		XUnmapWindow( QX11Info::display(), window );
		//XMoveResizeWindow(QX11Info::display(), window, 0, 0, QApplication::desktop()->size().width(), QApplication::desktop()->size().height() - TAB_BUTTON_HEIGHT);

		XMapRaised(QX11Info::display(), window );

		X11Info::setInputFocus( window );

		XSync(QX11Info::display(), false);
	}
}

void X11Info::showWindow( Window window )
{
	XMapRaised(QX11Info::display(), window );

	X11Info::setInputFocus( window );

	XSync(QX11Info::display(), false);
}

#endif
