#ifndef X11INFO_H
#define X11INFO_H

#include <QObject>

#ifdef Q_OS_LINUX

#include <QProcess>
#include <QFile>
#include <QList>
#include <QIcon>
#include <QX11Info>
#include <QWidget>
#include <QDesktopWidget>
#include <QApplication>
#include <QGuiApplication>
#include <QScreen>
#include <QPair>

#include <X11/X.h>

#include "constants.h"

class X11Info
{
	public:
		X11Info();

		static QList<long long> getVisibleWindows();
		static QList<Window> getProcessWindows(Q_PID pid);
		static Q_PID getWindowPid(Window window);
		static QList<Window> getAllChildren(Window window);
		static long getRailWindow( Window window );
		static QString getWindowTextProperty(Window window, QString property);
		static QString getWindowName(Window window);
		static QIcon getWindowIcon(Window window);
		static QString getWindowType(Window window);
		static int getWindowMapState(Window window);
		static bool getWindowSaveUnder(Window window);
		static QList<Window> getChildren(Window window);
		static Window getParent(Window window);
		static void showWindowsTree(Window window, int depth);
		static void showWindowInfo(Window window, int depth);
		static QSize getWindowMaxSize(Window window);
		static QSize getWindowMinSize(Window window);
		static QSize getWindowBaseSize(Window window);
		static QSize getWindowPreferredSize(Window window);
		static bool isValidWindow( Window window );
		static QPair<QString,QString> getWindowClass(Window window);

		static void showWindow( Window window );
		static void setWorkArea(QRect rect);
		static void setDesktopViewport(int x, int y);
		static void setDesktopGeometry(int w, int h);
		static void setWindowPosition(Window window, int x, int y);
		static void setWindowDock(Window window);

		static void sendKeyPressEvent( Window window, int modifiers, int keysym );
		static void setInputFocus( Window window );
		static void closeXorgWindow( Window window );
		static void closeWinWindow( Window window );
		static void killXorgWindow( Window window );
		static void killWinWindow( Window window );
		static void disconnectRdpWindow( Window window );
		static void attachWindow( QWidget * container, Window window );
		static void attachWindow( Window window );
		static void moveToBack(Window window);

		static void paintRectangle(Window window, QRect rect, char * color);
};

#endif

#endif
