#include "placesmodel.h"

Place::Place( QIcon icon, QIcon fileManagerIcon, QString name, QString location )
{
	this->icon = icon;
	this->fileManagerIcon = fileManagerIcon;
	this->name = name;
	this->location = location;
}

bool Place::validate()
{
	if ( QFile::exists( Settings::homePath() + QDir::separator() + location) || QFile::exists( location ) )
	{
		return true;
	} else {
		return QDir( Settings::homePath() ).mkdir( location );
	}
}

PlacesModel::PlacesModel(QObject * parent, bool isFileManager, QFont placesFont) : QStandardItemModel( parent )
{
	this->placesFont = placesFont;
	this->isFileManager = isFileManager;
}

PlacesModel::~PlacesModel()
{
	mutex.lock();
	clear();
	mutex.unlock();
}

void PlacesModel::updatePlaces(RemoteClientManager * remoteClientManager)
{
	mutex.lock();
	clear();
	Place * place;
#ifdef Q_OS_WIN
	place = new Place( QIcon(":/icons/64x64/folder-home.png"), QIcon(":/icons/64x64/home.png"), qgetenv( "USERNAME" ) );
#else
	place = new Place( QIcon(":/icons/64x64/folder-home.png"), QIcon(":/icons/64x64/home.png"), qgetenv( "USER" ) );
#endif
	appendItem( place );

	if ( remoteClientManager != nullptr )
	{
		for (int i = 0; i < remoteClientManager->getNames().count(); i++)
		{
			if ( Settings::homePath().toLower() != remoteClientManager->getDrives().at(i).toLower() )
			{
				QString remotePlaceName = tr("LOCAL > ") + remoteClientManager->getDrives().at(i);
				QString remotePlaceLocation = remoteClientManager->getDrives().at(i);
				place = new Place( QIcon(":/icons/64x64/folder-shared.png"), QIcon(":/icons/64x64/shared.png"), remotePlaceName, remotePlaceLocation);
				appendItem( place );
			}
		}
	}

	place = new Place( QIcon(":/icons/64x64/folder-documents.png"), QIcon(":/icons/64x64/documents.png"), tr("Documents"), "Documents" );
	if ( place->validate() )
	{
		appendItem( place );
	}

	place = new Place( QIcon(":/icons/64x64/folder-downloads.png"),  QIcon(":/icons/64x64/downloads.png"), tr("Downloads"), "Downloads" );
	if ( place->validate() )
	{
		appendItem( place );
	} else {
		delete place;
	}

	place = new Place( QIcon(":/icons/64x64/folder-desktop.png"),  QIcon(":/icons/64x64/desktop.png"), tr("Desktop"), "Desktop" );
	if ( place->validate() )
	{
		appendItem( place );
	} else {
		delete place;
	}

	place = new Place( QIcon(":/icons/64x64/folder-videos.png"),  QIcon(":/icons/64x64/videos.png"), tr("Videos"), "Videos" );
	if ( place->validate() )
	{
		appendItem( place );
	} else {
		delete place;
	}

	place = new Place( QIcon(":/icons/64x64/folder-pictures.png"),  QIcon(":/icons/64x64/pictures.png"), tr("Pictures"), "Pictures" );
	if ( place->validate() )
	{
		appendItem( place );
	} else {
		delete place;
	}

	place = new Place( QIcon(":/icons/64x64/folder-music.png"),  QIcon(":/icons/64x64/music.png"), tr("Music"), "Music" );
	if ( place->validate() )
	{
		appendItem( place );
	} else {
		delete place;
	}

	if (QFile::exists( Settings::homePath() + QDir::separator() + DROPBOX_HOME))
	{
		place = new Place( QIcon(":/icons/64x64/folder-dropbox.png"),  QIcon(":/icons/64x64/dropbox.png"), tr("Dropbox"), DROPBOX_HOME );
		appendItem( place );
	}

	if (QFile::exists( Settings::homePath() + QDir::separator() + GOOGLEDRIVE_HOME))
	{
		place = new Place( QIcon(":/icons/64x64/folder-googledrive.png"),  QIcon(":/icons/64x64/googledrive.png"), tr("Google Drive"), GOOGLEDRIVE_HOME );
		appendItem( place );
	}

	if (QFile::exists( MEDIA_DIR ))
	{
		place = new Place( QIcon(":/icons/64x64/folder-localhome.png"),  QIcon(":/icons/64x64/localhome.png"), tr("Removable Devices"), MEDIA_DIR );
		appendItem( place );
	}

	mutex.unlock();
}

void PlacesModel::appendItem(Place * place)
{
	QIcon icon;
	if ( isFileManager )
	{
		icon = place->fileManagerIcon;
	} else {
		icon = place->icon;
	}
	QStandardItem * item = new QStandardItem(icon, place->name);
	item->setEditable( false );
	item->setFont( placesFont );
	this->appendRow( item );
	places[place->name] = place;
}

Place * PlacesModel::getItem(QModelIndex index)
{
	QString name = index.data( Qt::DisplayRole ).toString();
	qDebug("getItem: %s", name.toUtf8().data());
	if ( places.contains( name ) )
	{
		qDebug("Location: %s", places[name]->location.toUtf8().data());
		return places[name];
	} else {
		return nullptr;
	}
}

void PlacesModel::removeIndex(QModelIndex index)
{
	QString name = index.data( Qt::DisplayRole ).toString();
	places.remove( name );
	removeRow( index.row() );
}

void PlacesModel::clear()
{
	QList<QString> keys = places.keys();
	for ( int i = 0; i < keys.count(); i++ )
	{
		QString item = keys[i];
		if ( ! item.isEmpty() )
		{
			Place * place = places[ item ];
			delete place;
		}
	}
	places.clear();
	QStandardItemModel::clear();
}
