#include "placeswidget.h"
#include "manager.h"

#include <QDir>

PlacesWidget::PlacesWidget(QWidget * parent) : IconView( parent, true )
{
	setFileSystemMode( false );

	init();
	updatePlaces();
}

void PlacesWidget::init()
{
	setOrientation( Qt::Horizontal );
	model = new PlacesModel;
	setModel( model );
	setEditTriggers(QAbstractItemView::NoEditTriggers);
	setStyleSheet(QString(	"QListView {"
								"	border-style: none;"
								"	background-color: transparent;"
								"	color: " FONTCOLOR ";"
							"}"));

	connect( this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(doubleClicked(QModelIndex)));
	connect( this, SIGNAL(clicked(QModelIndex)), this, SLOT(doubleClicked(QModelIndex)));
	connect( this, SIGNAL(indexesMoved(QModelIndexList)), this, SLOT(saveIconPositions()));
}

void PlacesWidget::updatePlaces()
{
	model->updatePlaces();
}

void PlacesWidget::doubleClicked(QModelIndex index)
{
	if ( index.isValid() )
	{
		Place * place = model->getItem( index );
		if ( place != 0 )
		{
			Application::launchFileManager( Settings::homePath() + QDir::separator() + place->location );
		}
	}
}

void PlacesWidget::loadIconPositions()
{
	QSettings settings(USER_CONFIG, QSettings::IniFormat);
	settings.beginGroup("Places");
	QStringList groupList = settings.allKeys();
	for ( int i = 0; i < groupList.count(); i++ )
	{
		if ( groupList[i].startsWith("PlacesWidget.") )
		{
			QString name = groupList.at(i).section( "PlacesWidget.", 1, 1 );
			for ( int k = 0; k < model->rowCount(); k++ )
			{
				Place * place = model->getItem( model->index( k, 0 ) );
				if ( place != 0 )
				{
					if ( place->name == name )
					{
						QPoint pos = settings.value( groupList[i] ).toPoint();
						QModelIndex existent = this->indexAt( pos );
						if ( existent.isValid() )
						{
							Place * movedPlace = model->getItem( existent );
							if ( movedPlace != 0 )
							{
								model->removeIndex( existent );
							}
							setPositionForIndex(  pos, model->index(k, 0) );
							if ( movedPlace != 0 )
							{
								model->appendItem( movedPlace );
							}
						} else {
							setPositionForIndex(  pos, model->index(k, 0) );
						}
					}
				}
			}
		}
	}
	settings.endGroup();
}

void PlacesWidget::saveIconPositions()
{
	QSettings settings(USER_CONFIG, QSettings::IniFormat);
	settings.beginGroup("Places");
	for ( int i = 0; i < model->rowCount(); i++ )
	{
		QModelIndex index = model->index( i, 0 );
		if ( index.isValid() )
		{
			Place * place = model->getItem( index );
			if ( place != 0 )
			{
				QString name = place->name;
				QPoint pos = rectForIndex( index ).topLeft();
				settings.setValue( "PlacesWidget." + name, QVariant(pos) );
			}
		}
	}
	settings.endGroup();
	settings.sync();
}
