#include "sharedesktopconfig.h"

ShareDesktopConfig::ShareDesktopConfig(QWidget * parent) : QDialog( parent )
{
    init();
}

void ShareDesktopConfig::init()
{
    layout = new QGridLayout(this);
    layout->setSpacing(10);
    layout->setMargin(20);

    backButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "back.png"), tr("Go back"), this);

	QFont font;
	font.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );
    QLabel * statusLabel = new QLabel(tr("Status: disabled"), this);
    statusLabel->setFont( font );

    enableButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "connect.png"), tr("Enable"), this);
    disableButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "power-pressed.png"), tr("Disable"), this);

    QLabel * passwordLabel = new QLabel(tr("Desktop sharing password:"), this);
    passwordLabel->setFont( font );
    password = new QLineEdit( "", this );
    password->setReadOnly( true );

    acceptButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "accept.png"), tr("Accept"), this);
    cancelButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "cancel.png"), tr("Cancel"), this);

    layout->addWidget(backButton, 0, 0);
    layout->addWidget(enableButton, 1, 0);
    layout->addWidget(disableButton, 1, 1);
    layout->addWidget(statusLabel, 2, 0, 1, 6);
    layout->addWidget(passwordLabel, 3, 0);
    layout->addWidget(password, 3, 1, 1, 5);
    layout->addWidget(acceptButton, 7, 4);
    layout->addWidget(cancelButton, 7, 5);

    for (int i = 0; i < 8; i++)
    {
        layout->setRowStretch(i, 1);
        layout->setColumnStretch(i, 1);
    }

    disableSharedDesktop();

    connect(enableButton, SIGNAL(clicked()), this, SLOT(enableSharedDesktop()));
    connect(disableButton, SIGNAL(clicked()), this, SLOT(disableSharedDesktop()));
    connect(backButton, SIGNAL(clicked()), this, SLOT(reject()));
    connect(acceptButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
}

void ShareDesktopConfig::enableSharedDesktop()
{
    QString connectionPassword = generatePassword(10);
    password->setText( connectionPassword );

    disableButton->setEnabled( true );
    enableButton->setEnabled( false );
}

void ShareDesktopConfig::disableSharedDesktop()
{
    password->clear();

    disableButton->setEnabled( false );
    enableButton->setEnabled( true );
}

QString ShareDesktopConfig::generatePassword(int passwordLength)
{
    const QString possibleCharacters("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789,.-;:_$@?¿=!*");

    QString password;
    for(int i = 0; i < passwordLength; i++)
    {
        int index = qrand() % possibleCharacters.length();
        QChar nextChar = possibleCharacters.at(index);
        password.append(nextChar);
    }

    return password;
}
