#ifndef THREADBUTTON_H
#define THREADBUTTON_H

#include <QPushButton>
#include <QString>
#include <QStringList>
#include <QIcon>
#include <QPixmap>
#include <QPainter>
#include <QFont>
#include <QWidget>
#include <QSize>
#include <QProcess>
#include <QMouseEvent>
#include <QMessageBox>

#include "constants.h"
#include "application.h"
#include "tooltip.h"

class ThreadButton : public QPushButton
{
	Q_OBJECT

	public:
#ifdef Q_OS_LINUX
		ThreadButton (QWidget *widget, int type, QPixmap pixmap = QPixmap(), QString label = QString(), QString text = QString(), HWND window = 0, Q_PID pid = 0, bool isRemote = false );
#endif
#ifdef Q_OS_WIN
		ThreadButton (QWidget *widget, int type, QPixmap pixmap = QPixmap(), QString label = QString(), QString text = QString(), HWND window = 0, unsigned long pid = 0, bool isRemote = false );
#endif
		~ThreadButton();

		static QSize buttonSize();
		int getType() { return type; };
		QPixmap getPixmap() { return pixmap; };
		void setPixmap(QPixmap pixmap);
		QString getLabel() { return label; };
		QString getText() { return text; };
		HWND getWinId() { return window; };
		void setWinId(HWND window) { this->window = window; };
		QWidget * getWidget() { return widget; };
#ifdef Q_OS_LINUX
		Q_PID getPid() { return pid; };
#endif
#ifdef Q_OS_WIN
		unsigned long getPid() { return pid; };
#endif
		void setRailWindow(HWND railWindow) { this->railWindow = railWindow; };
		HWND getRailWindow() { return railWindow; };

		void setRemoteServer(QString remoteServer) { this->remoteServer = remoteServer; };
		QString getRemoteServer() { return remoteServer; };

		int getIndex() { return index; };
		void setIndex(int index) { this->index = index; };
		int isRemoteApp() { return remote; };
		QPixmap getPreview() { return preview; };
		void setPreview(QPixmap preview) { this->preview = preview; };
		void setSelected(bool selected = true);

		void setLeftClickWidget( QWidget * widget ) { this->leftClickWidget = widget; };
		void setRightClickWidget( QWidget * widget ) { this->rightClickWidget = widget; };

	public slots:
		void closeButton();
		void closeApplication();
		void terminateApplication();

	signals:
		void threadButtonDoubleClicked();
		void threadButtonClicked( int index );
		void threadButtonClicked( HWND );
		void threadButtonClicked( QWidget * widget, bool = false );
		void closeThreadButton( int index );

	private slots:
		void buttonClicked();
		void mousePressEvent(QMouseEvent *e);
		void mouseDoubleClickEvent(QMouseEvent *e);

	private:
		bool event(QEvent *event);
		void paintEvent ( QPaintEvent * e);
		void forceRepaint();

		bool selected;
		QWidget * widget;
		QWidget * rightClickWidget;
		QWidget * leftClickWidget;
		QString label;
		QString text;
		QPixmap pixmap;
		QPixmap preview;
		int index;
		int type;
		HWND window;
		HWND railWindow;
		QString remoteServer;
		ToolTip * tip;
#ifdef Q_OS_LINUX
		Q_PID pid;
#endif
#ifdef Q_OS_WIN
		unsigned long pid;
#endif
		bool remote;
		bool active;
};

#endif
