#include "threadwidget.h"

#include <QDir>

ThreadWidget::ThreadWidget( QWidget * parent, bool forThreadManager ) : QFrame( parent )
{
	previewButton = 0;
	this->forThreadManager = forThreadManager;
	setFrameShape( QFrame::StyledPanel );

	if ( ! forThreadManager )
	{
		setStyleSheet( QString("background-color: %1").arg(BACKGROUNDCOLOR) );
	}

	layout = new QGridLayout;
	layout->setContentsMargins(2, 2, 2, 2);
	layout->setSpacing( 2 );

	name = new QLabel;
	text = new QLabel;

	closeButton = new CloseButton( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "cancel.png"), NULL );
	connect(closeButton, SIGNAL(clicked()), this, SLOT(closeButtonClickedSlot()));

	layout->addWidget( closeButton, 0, 1, Qt::AlignRight | Qt::AlignTop );

	if ( forThreadManager )
	{
		killButton = new CloseButton( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "terminate.png"), NULL );
		connect(killButton, SIGNAL(clicked()), this, SLOT(killButtonClickedSlot()));
		layout->addWidget( killButton, 1, 1, Qt::AlignRight | Qt::AlignTop );

		statusLabel = new QLabel;
		QFont font;
		font.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );
		QPalette palette;
		palette.setColor( QPalette::WindowText, QColor(Qt::darkGreen) );
		statusLabel->setText( tr("Running") );
		statusLabel->setFont( font );
		statusLabel->setPalette( palette );
		layout->addWidget( statusLabel, 2, 0, Qt::AlignCenter );

		previewButton = new QPushButton;
		previewButton->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
		connect( previewButton, SIGNAL(clicked()), this, SLOT(clicked()) );
		layout->addWidget( name, 0, 0, Qt::AlignCenter | Qt::AlignTop );
		layout->addWidget( text, 1, 0, Qt::AlignCenter | Qt::AlignTop );
		layout->addWidget( previewButton, 3, 0 );
	} else {
		previewButton = 0;
		layout->addWidget( name, 0, 0, Qt::AlignLeft | Qt::AlignTop );
		layout->addWidget( text, 1, 0, Qt::AlignLeft | Qt::AlignTop );
	}

	setLayout( layout );

    threadButton = 0;
}

void ThreadWidget::setThreadButton(ThreadButton * threadButton)
{
/*
	if ( previewButton != 0 )
		previewButton->setFixedSize( previewButton->size() );
*/
    this->threadButton = threadButton;
    if (threadButton == 0)
		return;

	QFont font;
	font.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );
	//name->setText( threadButton->getLabel() );
	name->setFont(font);
	//text->setText( threadButton->getText() );
	if (threadButton->getType() <= MENUBUTTON)
		closeButton->setHidden( true );
	else
		closeButton->setHidden( false );
    threadButton->setSelected( true );
	if ( previewButton != 0 )
	{
		QVBoxLayout * previewButtonLayout = new QVBoxLayout;
		previewButton->setLayout( previewButtonLayout );
		previewButton->setFlat( true );
		preview = new QLabel;
		preview->setScaledContents( true );
		previewButtonLayout->addWidget( preview );
		preview->setPixmap( threadButton->getPreview() );
	}
}

ThreadButton * ThreadWidget::getThreadButton()
{
    return threadButton;
}

void ThreadWidget::closeButtonClickedSlot()
{
	emit closeButtonClicked( threadButton );
}

void ThreadWidget::killButtonClickedSlot()
{
	
	int res = QMessageBox::warning ( NULL, tr("Terminate application"),
								tr("You are going to terminate the application.\n"
								"All unsaved data will be lost.\n"
								"Are you sure you want to do this?"),
								QMessageBox::Yes | QMessageBox::No,
								QMessageBox::No );

	if ( res == QMessageBox::Yes )
	{
		QFont font;
		font.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );
		QPalette palette;
		palette.setColor( QPalette::WindowText, QColor(Qt::black) );
		statusLabel->setText( tr("Terminating application") );
		statusLabel->setFont( font );
		statusLabel->setPalette( palette );

		emit killButtonClicked( threadButton );
	}
}

void ThreadWidget::clicked()
{
    emit clicked( threadButton );
}

void ThreadWidget::setName( QString threadName )
{
    name->setText( threadName );
	setToolTip(name->text() + "\n" + text->text());
}

void ThreadWidget::setText( QString threadText )
{
    text->setText( threadText );
	setToolTip(name->text() + "\n" + text->text());
}
