#ifndef TOUCHCONFIG_H
#define TOUCHCONFIG_H

#include <QObject>
#include <QMessageBox>
#include <QApplication>
#include <QDialog>
#include <QProgressDialog>
#include <QWidget>
#include <QGridLayout>
#include <QLineEdit>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QDir>
#include <QLabel>
#include <QPushButton>
#include <QComboBox>
#include <QSpinBox>
#include <QProcess>
#include <QTimer>

#include "configurationdialog.h"
#include "settings.h"
#include "lwclient.h"

#define DEVICETYPE Qt::UserRole + 1

class TouchConfig : public QDialog, public ConfigurationDialog
{
	Q_OBJECT

	public:
		TouchConfig(QWidget * parent);

		static bool isAvailable();
		void init();
		QIcon icon() { return QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "touchconfig.png"); };
		QString name() { return tr("Touch screen configuration"); };
		QDialog * widget() { return this; };

	private slots:
		void calibrateBackgroundSlot();
		void calibrateSlot();
		void calibrateFinished();

	private slots:
		void loadSettings();

	private:
		QString getMacAddress();
		bool checkUsb(QString vendor, QStringList products);
		bool checkUsb(QString vendor, QString product);

		QGridLayout * layout;

		QComboBox * devices;
		QPushButton * calibrate;
		QPushButton * refresh;
		QLabel * background;
		QProcess * calibrateProcess;
};

#endif
