#include "userconfig.h"

UserConfig::UserConfig(QWidget *parent ) : QDialog( parent )
{
	setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	//setMenuWidget( parent );
	init ();
}

void UserConfig::init()
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	QFont font;
	font.setFamily( Settings::getUserValue("BoldFont", LIBREWORK_FONT_BOLD).toString() );

	QLabel * usernameLabel = new QLabel(tr("User name:"), this);
	usernameLabel->setMinimumWidth( 150 );
	usernameLabel->setFont( font );
	username = new QLineEdit(Settings::getFullUserName() , this);
	username->setMinimumWidth( 200 );
	username->setReadOnly( true );

	QLabel * passwordLabel = new QLabel(tr("Actual password:"), this);
	passwordLabel->setFont( font );
	password = new QLineEdit( "", this );
	password->setEchoMode( QLineEdit::Password );

	QLabel * newPasswordLabel = new QLabel(tr("New password:"), this);
	newPasswordLabel->setFont( font );
	newPassword = new QLineEdit( this );
	newPassword->setEchoMode( QLineEdit::Password );

	QLabel * repeatPasswordLabel = new QLabel(tr("Repeat password:"), this);
	repeatPasswordLabel->setFont( font );
	repeatPassword = new QLineEdit( this );
	repeatPassword->setEchoMode( QLineEdit::Password );


	acceptButton = new QPushButton( tr("Accept"), this);
	acceptButton->setFlat( true );
	acceptButton->setStyleSheet(STANDARDBUTTONSTYLE);
	acceptButton->setMinimumSize( 100, 24 );
	cancelButton = new QPushButton( tr("Cancel"), this);
	cancelButton->setFlat( true );
	cancelButton->setStyleSheet(STANDARDBUTTONSTYLE);
	cancelButton->setMinimumSize( 100, 24 );

	layout->addWidget(usernameLabel, 1, 0);
	layout->addWidget(username, 1, 1, 1, 5);
	layout->addWidget(passwordLabel, 2, 0);
	layout->addWidget(password, 2, 1, 1, 5);
	layout->addWidget(newPasswordLabel, 3, 0);
	layout->addWidget(newPassword, 3, 1, 1,5);
	layout->addWidget(repeatPasswordLabel, 4, 0);
	layout->addWidget(repeatPassword, 4, 1, 1, 5);
	layout->addWidget(acceptButton, 5, 4);
	layout->addWidget(cancelButton, 5, 5);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect(acceptButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
}

void UserConfig::accept()
{
	if ( newPassword->text().length() == 0 )
	{
		QMessageBox::critical ( this, tr("Incorrect password" ), tr("The new password must not be empty"));
		return;
	}

	if ( newPassword->text() != repeatPassword->text() )
	{
		QMessageBox::critical ( this, tr("Password mismatch"), tr("The repeat password must be equal to the new password"));
		return;
	}
#ifdef Q_OS_WIN
    if (password->text().length() == 0)
    {
        QMessageBox::critical ( this, tr("Incorrect password" ), tr("The old password must not be empty"));
        return;
    }

    DWORD res = NetUserChangePassword( NULL, NULL, (const wchar_t *)password->text().utf16(), (const wchar_t *)newPassword->text().utf16());

    switch (res)
    {
    case ERROR_INVALID_PARAMETER: // Shows this weird result code when everything is ok
    case NERR_Success:
        QMessageBox::information ( this, tr("Password change succesful"), tr("The password has been changed."));
        Settings::setPassword( newPassword->text() );
        password->setText( newPassword->text() );
        newPassword->setText( "" );
        repeatPassword->setText( "" );
        QDialog::accept();
        break;
    case ERROR_INVALID_PASSWORD:
        QMessageBox::critical ( this, tr("Cannot change password" ), tr("The old password you provided is invalid"));
        return;
        break;
    case NERR_UserNotFound:
    case NERR_InvalidComputer:
    case ERROR_ACCESS_DENIED:
    case NERR_NotPrimary:
        QMessageBox::critical ( this, tr("Cannot change password" ), tr("Not enough permissions to change password"));
        return;
        break;
    case NERR_PasswordTooShort:
        QMessageBox::critical ( this, tr("Cannot change password" ), tr("The password you provided is too short or has been used previously"));
        return;
        break;
    default:
        QMessageBox::critical ( this, tr("Cannot change password" ), tr("There was a problem trying to change password. Contact the administrator"));
		qDebug("Error: %d", (int)res);
        return;
    }

#endif

#ifdef Q_OS_LINUX
	/* DIRTY HACK: Doing password set via lwserver
	   This is not correct, as it forces to have librework-desktop
	   on the same server as samba. See below the correct
	   approach via LDAP
	   Better via smbpasswd:
	   smbpasswd -r <AD server ip address> -U <username> -s
	*/

	int port = Settings::getValue( "Server/LibreworkServerPort" ).toInt();
	QString address = Settings::getValue( "Server/LibreworkServerAddress" );

	QTcpSocket * tcpSocket = new QTcpSocket(this);

	setCursor(Qt::WaitCursor);

	tcpSocket->connectToHost( address, port, QIODevice::ReadWrite );

	if (tcpSocket->waitForConnected( TIMEOUT ))
	{
		qDebug("Connected");
		// Authentication
        QString message = "login?" + Settings::getFullUserName() + "&&" + Settings::getPassword() + "\n";
		tcpSocket->write(message.toUtf8().data());
		tcpSocket->waitForReadyRead(5000);
		QString response = tcpSocket->readAll();
		if ( response.startsWith( "Error" ) )
		{
			QMessageBox::critical ( this, tr("Invalid password"), tr("The actual password you have entered is incorrect, or you don't have access to the LDAP server"));
		} else {
			message = "modify_user_password?" + username->text() + "&" + password->text() + "&" + newPassword->text();
			tcpSocket->write(message.toUtf8().data());
			tcpSocket->waitForReadyRead();
			response = tcpSocket->readAll();
			if ( response.startsWith("Error") )
			{
				QMessageBox::critical ( this, tr("Unauthorized"), tr("You have no permission to change the password"));
			} else {
				QMessageBox::information ( this, tr("Password change succesful"), tr("The password has been changed."));
				Settings::setPassword( newPassword->text() );
				password->setText( newPassword->text() );
				newPassword->setText( "" );
				repeatPassword->setText( "" );
			}
		}
		tcpSocket->close();
	} else {
		QMessageBox::critical ( this, tr("Connection error"), tr("Cannot connect to server"));
	}
	delete tcpSocket;
#endif

	setCursor(Qt::ArrowCursor);

/* Must be done as this... in a near future
	LdapInfo * ldap = new LdapInfo(Settings::getValue("LdapSuffix"), Settings::getValue("LdapServer"), Settings::ldapServerPort() );

	if ( ! ldap->authenticate( Settings::getUserName(), password->text() ) )
	{
		QMessageBox::critical ( this, tr("Invalid password"), tr("The actual password you have entered is incorrect, or you don't have access to the LDAP server"));
	} else {
		if ( ! ldap->setValue( "cn=Users,cn=" + Settings::getUserName(), "unicodePwd", newPassword->text() ) )
		{
			QMessageBox::critical ( this, tr("Unauthorized"), tr("You have no permission to change the password"));
		}
		ldap->close();
		if ( dontclose )
		{
			menuWidget()->setPrincipalWidget();
		} else {
			QDialog::accept();
		}
	}

	delete ldap;
*/
}

void UserConfig::reject()
{
	password->clear();
	newPassword->clear();
	repeatPassword->clear();
	username->setFocus();
}
