#include "winaction.h"

#ifdef Q_OS_LINUX
#include <QX11Info>
#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#endif

#ifndef MIN_ALL
#define MIN_ALL 419
#endif
#ifndef MIN_ALL_UNDO
#define MIN_ALL_UNDO 416
#endif

void WinAction::sendClientMessage(uint32_t mask, xcb_window_t destination, xcb_window_t window, xcb_atom_t message, const uint32_t data[])
{
	xcb_client_message_event_t event;
	event.response_type = XCB_CLIENT_MESSAGE;
	event.format = 32;
	event.sequence = 0;
	event.window = window;
	event.type = message;

	for (int i = 0; i < 5; i++)
	{
		event.data.data32[i] = data[i];
	}

	xcb_send_event(xcb_connect(nullptr, nullptr), false, destination, mask, (const char *) &event);
}

void WinAction::setInputFocus( HWND id )
{
#ifdef Q_OS_WIN
	SetFocus( id );
#endif
#ifdef Q_OS_LINUX
	XSetInputFocus(QX11Info::display(), (Window) id, RevertToParent, CurrentTime);
#endif
}

void WinAction::setWindowPosition(HWND id, int x, int y)
{
#ifdef Q_OS_WIN
	SetWindowPos( id, static_cast<HWND>(HWND_TOPMOST), x, y, 0, 0, SWP_SHOWWINDOW|SWP_NOSIZE|SWP_NOMOVE );
#endif
#ifdef Q_OS_LINUX
	XMoveWindow( QX11Info::display(), (Window)id, x, y);
	//Window ret;
	//XTranslateCoordinates( QX11Info::display(), window, XDefaultRootWindow(QX11Info::display()), 0, 0, &x, &y, &ret );
#endif
}

void WinAction::setDesktopWindow( HWND id )
{
#ifdef Q_OS_WIN
	setWindowAlwaysInactive( id );
#endif
#ifdef Q_OS_LINUX
	Display * display  = QX11Info::display();
	Atom tmp = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DESKTOP", False);
	XChangeProperty(display,
		id,
		XInternAtom(display, "_NET_WM_WINDOW_TYPE", False),
		XA_ATOM ,
		32,
		PropModeReplace,
		(unsigned char *)&tmp, 1);

	QSize geometry = Manager::getDesktopSize();
	uint32_t data[2] = { uint32_t(geometry.width()), uint32_t(geometry.height())};
	XChangeProperty(display,
		id,
		XInternAtom(display, "_NET_DESKTOP_GEOMETRY", False),
		XA_ATOM ,
		32,
		PropModeReplace,
		(unsigned char *)data, 2);
	//Atom desktopGeometry = XInternAtom(display, "_NET_DESKTOP_GEOMETRY", False);
	//sendClientMessage( SubstructureRedirectMask|SubstructureNotifyMask, XDefaultRootWindow(display), XDefaultRootWindow(display), desktopGeometry, data);
#endif
}

void WinAction::setDockWindow( HWND id )
{
	qDebug("Setting dock window");
#ifdef Q_OS_WIN
	/* For windows 10 (Not working!)
	APPBARDATA dock;
	dock.cbSize = sizeof( APPBARDATA );
	dock.hWnd = id;
	dock.lParam = TRUE;
	SHAppBarMessage(ABM_NEW, &dock);
	SHAppBarMessage(ABM_ACTIVATE, &dock);*/
	Q_UNUSED( id );
	// Create work area
	RECT rect;
	QSize desktopSize = Manager::getDesktopSize();
	rect.top = 0;
	rect.bottom = desktopSize.height();
	rect.left = 0;
	rect.right = desktopSize.width();
	bool result = SystemParametersInfo(SPI_SETWORKAREA, 0, &rect, SPIF_UPDATEINIFILE);
	SystemParametersInfo(SPI_SETWORKAREA, 0, &rect, 0);
	SystemParametersInfo(SPI_SETWORKAREA, 0, &rect, SPIF_SENDCHANGE);
	if (!result)
	{
		qDebug() <<  "Cannot change work area";
	} else {
		qDebug("Work area set to %dx%d. Re-ubicating windows", desktopSize.width(), desktopSize.height());
		LPCWSTR className = reinterpret_cast<LPCWSTR>("Shell_TrayWnd");
		HWND lHwnd = FindWindow( className, nullptr);
		LPARAM Zero = 0;
		SendMessage(lHwnd, WM_COMMAND, MIN_ALL, Zero);
		QThread::msleep(10);
		SendMessage(lHwnd, WM_COMMAND, MIN_ALL_UNDO, Zero);
		QThread::msleep(10);
	}
	//SetWindowPos(id, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
#endif
#ifdef Q_OS_LINUX
	Display * display  = QX11Info::display();

	// Define type of window as dock
	Atom tmp = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DOCK", False);
	XChangeProperty(display,
		id,
		XInternAtom(display, "_NET_WM_WINDOW_TYPE", False),
		XA_ATOM,
		32,
		PropModeReplace,
		(unsigned char *)&tmp, 1);

	// Show on all virtual desktops
	/*long data[1] = { 0xFFFFFFFF };
	XChangeProperty(display,
		id,
		XInternAtom(display, "_NET_WM_DESKTOP", False),
		XA_CARDINAL,
		32,
		PropModeReplace,
		(unsigned char *) &data, 1);
	*/

	// left, right, top, bottom,
	// left_start_y, left_end_y, right_start_y, right_end_y, top_start_x, top_end_x, bottom_start_x, bottom_end_x
	// [0, 0, bar_size, 0, 0, 0, 0, 0, x, x+width-1, 0, 0]
	// QApplication::primaryScreen()->size().height() - TAB_BUTTON_HEIGHT

	int insets[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	insets[0] = 0;
	insets[1] = Manager::getDesktopSize().width();
	insets[2] = Manager::getDesktopSize().height();
	insets[3] = Manager::getDesktopSize().height() + TAB_BUTTON_HEIGHT;
	insets[4] = Manager::getDesktopSize().height();
	insets[5] = Manager::getDesktopSize().height() + TAB_BUTTON_HEIGHT;
	insets[6] = Manager::getDesktopSize().height();
	insets[7] = Manager::getDesktopSize().height() + TAB_BUTTON_HEIGHT;
	insets[8] = 0;
	insets[9] = Manager::getDesktopSize().width();
	insets[10] = 0;
	insets[11] = Manager::getDesktopSize().width();

	XChangeProperty(display,
		XDefaultRootWindow(QX11Info::display()),
		XInternAtom(display, "_NET_WM_STRUT", False),
		XA_CARDINAL ,
		32,
		PropModeReplace,
		(unsigned char *)&insets, 4);

	/*XChangeProperty(display,
		XDefaultRootWindow(QX11Info::display()),
		XInternAtom(display, "_NET_WM_STRUT_PARTIAL", False),
		XA_CARDINAL ,
                32,
		PropModeReplace,
		(unsigned char *)&insets, 12);
	*/

	// Define work area
	/*
	int workArea[4] = { 0, 0, Manager::getDesktopSize().width(), Manager::getDesktopSize().height() };
	XChangeProperty(display,
		XDefaultRootWindow(QX11Info::display()),
		XInternAtom(display, "_NET_WORKAREA", False),
		XA_CARDINAL ,
                32,
		PropModeReplace,
		(unsigned char *)&workArea, 4);
	*/

	// Move dock to its place
	locateWindow( id, 0, Manager::getDesktopSize().height(), Manager::getDesktopSize().width(), TAB_BUTTON_HEIGHT );
#endif
}

void WinAction::locateWindow( HWND id, int x, int y, int width, int height)
{
#ifdef Q_OS_LINUX
	XMoveResizeWindow(QX11Info::display(), id, x, y, width, height);
#endif
}

void WinAction::showWindow( HWND id )
{
#ifdef Q_OS_WIN
	if ( IsIconic( id ) )
	{
		ShowWindow( id, SW_RESTORE );
	} else {
		ShowWindow( id, SW_SHOW );
	}
	BringWindowToTop( id );
#endif
#ifdef Q_OS_LINUX
	XEvent event = { 0 };
	event.xclient.type = ClientMessage;
	event.xclient.serial = 0;
	event.xclient.send_event = True;
	event.xclient.message_type = XInternAtom( QX11Info::display(), "_NET_ACTIVE_WINDOW", False);
	event.xclient.window = id;
	event.xclient.format = 32;
	XSendEvent( QX11Info::display(), XDefaultRootWindow(QX11Info::display()), False, SubstructureRedirectMask | SubstructureNotifyMask, &event );

	XSync(QX11Info::display(), false);
	XMapRaised(QX11Info::display(), (Window) id );
	qApp->processEvents();
	setInputFocus( (Window) id );
#endif
}

void WinAction::minimizeWindow( HWND id )
{
#ifdef Q_OS_WIN
	ShowWindow( id, SW_FORCEMINIMIZE );
#endif
#ifdef Q_OS_LINUX
	XIconifyWindow(QX11Info::display(), (Window) id, 0);
#endif
}

void WinAction::moveToBack( HWND id )
{
#ifdef Q_OS_WIN
	SetWindowPos( id, HWND_BOTTOM, 0, 0, 0, 0, SWP_SHOWWINDOW|SWP_NOSIZE|SWP_NOMOVE);
#endif
#ifdef Q_OS_LINUX
	XLowerWindow(QX11Info::display(), (Window)id);
	XCirculateSubwindowsDown(QX11Info::display(), (Window)id);
#endif
}

void WinAction::moveToFront( HWND id )
{
	WinAction::showWindow( id );
}

void WinAction::setRootWindowBackgroundColor( QColor color )
{
#ifdef Q_OS_WIN
	// First, remove background wallpaper
	if ( SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (PVOID)"", SPIF_SENDCHANGE | SPIF_UPDATEINIFILE) == S_OK )
	{
		int aiElements[1] = {COLOR_BACKGROUND};
		BYTE red = color.red();
		BYTE green = color.green();
		BYTE blue = color.blue();
		DWORD aColors[1] = {RGB( red, green, blue)};
		SetSysColors(1, aiElements, aColors );
	}
#endif
#ifdef Q_OS_LINUX
	XSetWindowBackground( QX11Info::display(), DefaultRootWindow(QX11Info::display()), (unsigned long int)color.rgb() );
	XClearWindow(QX11Info::display(), DefaultRootWindow(QX11Info::display()));
	XFlush(QX11Info::display());
#endif
}

void WinAction::closeWindow( HWND id )
{
#ifdef Q_OS_WIN
	PostMessage(id, WM_SYSCOMMAND, SC_CLOSE, 0);
#endif
#ifdef Q_OS_UNIX
		if (id != 0)
		{
				XEvent event;
				event.xclient.type         = ClientMessage;
				event.xclient.window       = id;
				event.xclient.message_type = XInternAtom(QX11Info::display(), "WM_PROTOCOLS", false);
				event.xclient.format       = 32;
				event.xclient.data.l[0]    = XInternAtom(QX11Info::display(), "WM_DELETE_WINDOW", false);
				event.xclient.data.l[1]    = CurrentTime;
				XSendEvent(QX11Info::display(), id, false, StructureNotifyMask, &event);
				XFlush(QX11Info::display());
		}
#endif
}

void WinAction::closeRemoteWindow( HWND id )
{
#ifdef Q_OS_UNIX
	if ( RDP_CLIENT == QString( "rdesktop" ) )
	{
		sendKeyPressEvent( id, ShiftMask, XK_F11 );
	}

	if ( RDP_CLIENT == QString( "xfreerdp" ) )
	{
		sendKeyPressEvent( id, 0, XK_Shift_L );
		sendKeyPressEvent( id, 0, XK_F11 );
	}
#endif
}

void WinAction::terminateWindow( HWND id )
{
#ifdef Q_OS_WIN
	PostMessage(id, WM_QUIT, 0, 0);
#endif
#ifdef Q_OS_UNIX
	XKillClient(QX11Info::display(), id);
	closeWindow( id );
//	XDestroyWindow(QX11Info::display(), id);
#endif
}

void WinAction::terminateRemoteWindow( HWND id )
{
#ifdef Q_OS_UNIX
	if ( RDP_CLIENT == QString( "rdesktop" ) )
	{
		sendKeyPressEvent( id, ShiftMask, XK_F12 );
	}
	if ( RDP_CLIENT == QString( "xfreerdp" ) )
	{
		sendKeyPressEvent( id, 0, XK_Shift_L );
		sendKeyPressEvent( id, 0, XK_F12 );
	}
#endif
}

void WinAction::sendKeyPressEvent( HWND id, int modifiers, int keysym )
{
#ifdef Q_OS_LINUX
	if (id != 0)
	{
		XKeyEvent event;

		event.display		= QX11Info::display();
		event.window		= id;
		event.root		= XDefaultRootWindow(QX11Info::display());
		event.subwindow		= None;
		event.time		= CurrentTime;
		event.x			= 1;
		event.y			= 1;
		event.x_root		= 1;
		event.y_root		= 1;
		event.same_screen	= True;
		event.keycode		= XKeysymToKeycode(QX11Info::display(), keysym);
		event.state		= modifiers;
		event.type		= KeyPress;

		XSendEvent(QX11Info::display(), id, true, KeyPressMask, (XEvent *)&event);
	}
#endif
#ifdef Q_OS_WIN
	QKeyEvent keyEvent( QEvent::None, keysym, (Qt::KeyboardModifiers)modifiers );
	keybd_event( keyEvent.nativeVirtualKey(), 0x45, KEYEVENTF_EXTENDEDKEY | 0, 0 );
	keybd_event( keyEvent.nativeVirtualKey(), 0x45, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0 );
#endif
}

void WinAction::setWindowAlwaysInactive(HWND id)
{
#ifdef Q_OS_WIN
	 SetWindowLong(id, GWL_EXSTYLE, GetWindowLong(id, GWL_EXSTYLE) | WS_EX_NOACTIVATE | WS_EX_APPWINDOW);
#endif
#ifdef Q_OS_LINUX
	 XSetWindowAttributes winattr;
	 winattr.override_redirect = 1;
	 XChangeWindowAttributes(QX11Info::display(), id, CWOverrideRedirect, &winattr);
#endif
}
