#include <QDir>
#include <QLabel>
#include <QApplication>

#include "wininfo.h"

#ifdef Q_OS_LINUX
#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#endif

#ifdef Q_OS_WIN
BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lparam)
{
	handleData * data = (handleData*)lparam;

	if ( data->process != nullptr ) // enum windows for that process
	{
		unsigned long processId = 0;
		GetWindowThreadProcessId(hWnd, &processId);
		if (data->process->processId() == processId)
		{
			data->windows += hWnd;
		}
		return true;
	} else { // enum all  windows
		/*if ( data->systemTray )
		{
			if ( WinInfo::isSystemTray(hWnd) )
			{
				data->windows += hWnd;
			}
		} else {*/
			if ( WinInfo::isVisible(hWnd) )
			{
				data->windows += hWnd;
			}
		/*}*/
		return true;
	}
}

BOOL CALLBACK FindWindowProc(HWND hWnd, LPARAM lparam)
{
	handleData * data = (handleData*)lparam;
	DWORD lpdwProcessId;
	GetWindowThreadProcessId(hWnd, &lpdwProcessId);
	if( lpdwProcessId == data->process->processId() )
	{
		data->first = hWnd;
		return FALSE;
	}
	return TRUE;
}

#endif

QPixmap WinInfo::toPixmap( HWND id )
{
	QPixmap ret;
#ifdef Q_OS_LINUX
	//XWindowAttributes xwa;
	//XGetWindowAttributes(QX11Info::display(), id, &xwa);
	QScreen * screen = qApp->primaryScreen();
	ret = screen->grabWindow( id );
#endif
	return ret;
}

QString WinInfo::getTitle( HWND id )
{
#ifdef Q_OS_WIN
	int maxString = 256;
	LPWSTR title = new WCHAR[maxString + 1];
	int result = GetWindowText(id, title, maxString);
	QString titleString = QString::fromWCharArray(title);
	delete[] title;
	if (result > 0)
	{
		return QString(titleString);
	} else {
		return QString();
	}
#endif
#ifdef Q_OS_LINUX
	QString res = getWindowTextProperty(id, "_NET_WM_NAME");

	if (res.isEmpty())
		res = getWindowTextProperty(id, "XA_WM_NAME");

	if (res.isEmpty())
		res = getWindowTextProperty(id, "WM_NAME");

	return res;
#endif
}

QPixmap WinInfo::getIcon( HWND id )
{
	QPixmap pixmap(ICON_DIR_64 + QString(QDir::separator()) + "openwith.png");

#if defined(Q_OS_WIN)
	// Retrieving the application icon
#ifndef GCL_HICON
#define GCL_HICON -14
#endif
#ifndef GCL_HICONSM
#define GCL_HICONSM
#endif
	HICON iconHandle = (HICON)SendMessage(id,WM_GETICON, ICON_BIG, 0);
	if(iconHandle == nullptr)
	{
		iconHandle = (HICON)SendMessage(id,WM_GETICON,ICON_SMALL2, 0);
	}

	if(iconHandle == nullptr)
	{
		iconHandle = (HICON)SendMessage(id,WM_GETICON,ICON_SMALL, 0);
	}

	if (iconHandle == nullptr)
	{
		iconHandle = (HICON)GetClassLongPtr(id, GCL_HICON);
	}

	if (iconHandle != nullptr)
	{
		pixmap = QtWin::fromHICON( iconHandle );
	}
#endif
#ifdef Q_OS_LINUX
	unsigned long nitems, bytesafter;
	unsigned char *data;
	int format;
	Atom type;
	Atom iconAtom = XInternAtom(QX11Info::display(), "_NET_WM_ICON", 1);

	// Icon width
	XGetWindowProperty(QX11Info::display(), id, iconAtom, 0, MAX_ICON_BYTESIZE, False, XA_CARDINAL,
						&type, &format, &nitems, &bytesafter,(unsigned char**) &data);

	if ( ! data )
	{
		//qDebug("Can't get icon data. Trying legacy xlib window icon.");
		XWMHints *hints;

		if ( (hints = XGetWMHints(QX11Info::display(), id)) )
		{
			if (hints->flags & IconPixmapHint)
			{
				//qDebug("IconPixmapHint found");
				bool mask = false;

				Window xr;
				XImage * xm;
				int  mx, my;
				unsigned int mw, mh, mb, md;
				if ( XGetGeometry( QX11Info::display(), hints->icon_mask, &xr, &mx, &my, &mw, &mh, &mb, &md ) )
				{
					xm = XGetImage( QX11Info::display(), hints->icon_mask, 0, 0, mw, mh, 0xffffffff, ZPixmap );
					if ( xm )
					{
						mask = true;
					}
				}

				XImage * xi;
				int  px, py;
				unsigned int pw, ph, pb, pd;
				if ( XGetGeometry( QX11Info::display(), hints->icon_pixmap, &xr, &px, &py, &pw, &ph, &pb, &pd ) )
				{
					xi = XGetImage( QX11Info::display(), hints->icon_pixmap, 0, 0, pw, ph, 0xffffffff, ZPixmap );
					if ( xi )
					{
						pixmap = pixmapFromXImage( xi, xm );
						XDestroyImage(xi);
					}
				}
				if ( mask )
				{
					XDestroyImage(xm);
				}
			}
			XFree( hints );
		}
		return pixmap;
	}

	pixmap = pixmapFromData((unsigned long int*) data, nitems );

	XFree( data );
#endif

	return pixmap;
}

#ifdef Q_OS_LINUX
QPixmap WinInfo::pixmapFromData(unsigned long int *data, int nitems)
{
	QPixmap pixmap;

	int offset = 0;
	unsigned int width = (unsigned int)data[0];
	unsigned int height = (unsigned int)data[1];

	unsigned int maxWidth = width;
	unsigned int maxHeight = height;
	int rightOffset = offset;

	while ( offset + width * height + 2 < (unsigned int) nitems  || (width == 0 || height == 0) )
	{
		offset = offset + 2 + width * height;
		width = (unsigned int)data[offset];
		height = (unsigned int)data[offset + 1];
		// Get best icon (max size)
		if (width > maxWidth && height > maxHeight )
		{
			rightOffset = offset;
			maxWidth = width;
			maxHeight = height;
		}
	}
	offset = rightOffset;
	width = maxWidth;
	height = maxHeight;

	if ( width == 0 || height == 0 || width * height + 2 + offset > (unsigned int)nitems)
	{
		return QPixmap(ICON_DIR_64 + QString(QDir::separator()) + "openwith.png");
	}

	QImage qimage((int)width, (int)height, QImage::Format_ARGB32);

	for(unsigned int y = 0; y < height; y++)
	{
		for(unsigned int x = 0; x < width; x++)
		{
			int index = offset + 2 + y * width + x;
			unsigned char * _data =(unsigned char*) (& data[index]);
			int r = _data[2];
			int g = _data[1];
			int b = _data[0];
			int a = _data[3];
			qimage.setPixelColor(x, y, QColor(r, g, b, a) );
		}
	}

	pixmap.convertFromImage(qimage);

	return pixmap;
}

QPixmap WinInfo::pixmapFromXImage(void *image, void *mask )
{
	XImage * _image = (XImage*) image;
	XImage * _mask = (XImage*) mask;

	QImage qimage( _image->width, _image->height, QImage::Format_ARGB32);

	int index = 0;
	int mIndex = 0;

	for(int y = 0; y < _image->height; y++)
	{
		for(int x = 0; x < _image->width; x++)
		{
			int r = ((int)((unsigned char)_image->data[index + 2]));
			int g = ((int)((unsigned char)_image->data[index + 1]));
			int b = ((int)((unsigned char)_image->data[index + 0]));
			int a;
			if (!((((unsigned)_mask->data[mIndex + x / 8]) >> (x % 8)) & 0x1))
			{
				a = 0;
			} else {
				a = 255;
			}

			qimage.setPixelColor(x, y, QColor( r, g, b, a ) );

			index = index + 4;
		}
		mIndex = mIndex + _mask->bytes_per_line;
	}

	QPixmap pixmap;
	pixmap.convertFromImage(qimage);

	return pixmap;
}
#endif

HWND WinInfo::getWindow(QProcess * process)
{
#ifdef Q_OS_LINUX
	Q_UNUSED( process );
	return -1;
#endif
#ifdef Q_OS_WIN
	handleData data;
	data.process = process;
	data.windows.clear();
	data.first = (HWND) -1;
	EnumWindows(FindWindowProc, (LPARAM)&data);
	return data.first;
#endif
}

bool WinInfo::isApplication( HWND id )
{
#ifdef Q_OS_WIN
	bool isChild = isWindowStyle(id, WS_CHILDWINDOW);
	bool dontActivate = isWindowExStyle(id, WS_EX_NOACTIVATE);
	bool isExApp = isWindowExStyle( id, WS_EX_APPWINDOW ) || isWindowExStyle(id, WS_EX_OVERLAPPEDWINDOW);

	return ! isChild && ! dontActivate && isExApp;
#endif
#ifdef Q_OS_LINUX
	return isWindowType( "_NET_WM_WINDOW_TYPE_NORMAL", id ) && ! isWindowType( "_NET_WM_WINDOW_TYPE_DOCK", id )
		&& ! hasState( "_NET_WM_STATE_SKIP_PAGER", id ) && ! hasState( "_NET_WM_STATE_SKIP_TASKBAR", id );
#endif
}

#ifdef Q_OS_LINUX
bool WinInfo::hasState( QString state, HWND id )
{
	bool ret = false;
	Atom wmState = XInternAtom(QX11Info::display(), "_NET_WM_STATE", True);
	Atom type;
	int format;
	unsigned long count, bytesAfter;
	unsigned char *properties = NULL;
	XGetWindowProperty(QX11Info::display(), (Window)id, wmState, 0, (~0L), False, AnyPropertyType, &type, &format, &count, &bytesAfter, &properties);
	for ( unsigned long i = 0; i < count; i++)
	{
		const Atom prop = (reinterpret_cast<ulong *>(properties))[i];
		if ( prop == XInternAtom(QX11Info::display(), state.toUtf8().data(), False) )
		{
			ret = true;
			i = count;
		}
	}
	XFree( properties );
	return ret;
}
#endif

bool WinInfo::isMinimized( HWND id )
{
#ifdef Q_OS_WIN
	return IsIconic( id );
#endif
#ifdef Q_OS_LINUX
	bool minimized = false;
	Atom wmState = XInternAtom(QX11Info::display(), "WM_STATE", True);
	Atom type;
	int format;
	unsigned long count, bytesAfter;
	unsigned char *data = NULL;
	XGetWindowProperty(QX11Info::display(), (Window)id, wmState, 0, LONG_MAX/*(~0L)*/, False, wmState, &type, &format, &count, &bytesAfter, &data);
	if ( data )
	{
		const ulong * prop = (reinterpret_cast<ulong *>(data));
		if ( *prop == 3 ) // Iconic = 3
		{
			minimized = true;
		}
		XFree( data );
	}
	/*for ( unsigned long i = 0; i < count; i++)
	{
		const Atom prop = (reinterpret_cast<ulong *>(properties))[i];
		if ( prop == XInternAtom(QX11Info::display(), "_NET_WM_STATE_HIDDEN", False) )
		{
			minimized = true;
			i = count;
		}
	}
	XFree( data );*/
	return minimized;
#endif
}

QList<HWND> WinInfo::getSystemTrayWindows()
{
	return getWindows( true );
}

QList<HWND> WinInfo::getApplicationWindows()
{
	QList<HWND> windows = getWindows( false );
	QList<HWND> ret;
	for ( int i = 0; i < windows.count(); i++ )
	{
#ifdef Q_OS_WIN
		if ( WinInfo::hasNoParent( windows[i] ) &&
			 ! WinInfo::isWindowExStyle( windows[i], WS_EX_TOOLWINDOW ) &&
			( WinInfo::isWindowExStyle( windows[i], WS_EX_APPWINDOW ) || WinInfo::isWindowStyle( windows[i], WS_POPUPWINDOW) ) )
#endif
#if defined(Q_OS_LINUX) || defined(Q_OS_FREEBSD)
		if (	! hasState( "_NET_WM_STATE_SKIP_PAGER", windows[i] ) &&
			! hasState( "_NET_WM_STATE_SKIP_TASKBAR", windows[i] ) )
#endif
		{
			ret += windows[i];
		}
	}
	return ret;
}

QList<HWND> WinInfo::getWindows(bool systemTrayWindows)
{
#ifdef Q_OS_WIN
	handleData data;
	data.process = NULL; // enum all visible windows
	data.windows.clear();
	data.systemTray = systemTrayWindows;
	//data.first = (HWND) -1;
	//EnumWindows(EnumWindowsProc, (LPARAM)&data);
	EnumChildWindows(GetDesktopWindow(), EnumWindowsProc, (LPARAM)&data );
	return data.windows;
#endif
#ifdef Q_OS_LINUX
	QList<HWND> ret;
	Atom a;
	if ( systemTrayWindows )
	{
		a = XInternAtom(QX11Info::display(), "_NET_CLIENT_LIST" , true);
	} else {
		a = XInternAtom(QX11Info::display(), "_NET_CLIENT_LIST" , true);
	}
	Atom actualType;
	int format;
	unsigned long numItems, bytesAfter;
	unsigned char *data = 0;

	int status = XGetWindowProperty( QX11Info::display(),
					XDefaultRootWindow(QX11Info::display()),
					a,
					0L,
					(~0L),
					false,
					AnyPropertyType,
					&actualType,
					&format,
					&numItems,
					&bytesAfter,
					&data);

	if (status >= Success && numItems)
	{
		long * array = (long*) data;
		for (unsigned int k = 0; k < numItems; k++)
		{
			HWND window = (HWND) array[k];
			ret.append( window );
		}
		XFree(data);
	}
	return ret;
#endif
}

HWND WinInfo::getTopMostWindow()
{
#ifdef Q_OS_WIN
	HWND foregroundWindow = GetForegroundWindow();
	return foregroundWindow;
#endif
#ifdef Q_OS_LINUX
	Window w;
	int revert_to;
	XGetInputFocus(QX11Info::display(), &w, &revert_to); // see man
	if (w == None)
	{
		Atom type;
		int format;
		unsigned long count, bytesAfter;
		unsigned char *data = NULL;
		Atom activeWindow = XInternAtom(QX11Info::display(), "_NET_ACTIVE_WINDOW", 1);
		int ret = XGetWindowProperty(QX11Info::display(),
			XDefaultRootWindow(QX11Info::display()),
			activeWindow, 0, 1, False, XA_CARDINAL,
			&type, &format, &count, &bytesAfter, &data);
		if ( ret == Success && count >= 3 )
		{
			long * array = (long*) data;
			w = (Window) array[2];
			XFree( data );
			return (HWND) w;
		} else {
			return -1;
		}
	}
	return (HWND) w;
#endif
}

#ifdef Q_OS_WIN
int WinInfo::isWindowStyle(HWND hWnd, long style)
{
	long gwlStyle = GetWindowLong(hWnd, GWL_STYLE);
	return gwlStyle & style;
}

int WinInfo::isWindowExStyle(HWND hWnd, long exStyle)
{
	long gwlExStyle = GetWindowLong(hWnd, GWL_EXSTYLE);
	return gwlExStyle & exStyle;
}

bool WinInfo::isVisible(HWND id)
{
	return isWindowStyle( id, WS_VISIBLE);
}
#endif

HWND WinInfo::getParent(HWND id)
{
#ifdef Q_OS_WIN
	HWND hParent = GetAncestor(id, GA_PARENT);
	if ( ! hParent )
	{
		return GetDesktopWindow();
	} else {
		return hParent;
	}
#endif
#ifdef Q_OS_UNIX
	Window root, parent, *children = NULL;
	unsigned int num_children;

	if( ! XQueryTree(QX11Info::display(), id, &root, &parent, &children, &num_children) )
	{
		return (HWND)XDefaultRootWindow(QX11Info::display());
	}

	if (children)
	{
		XFree((char *)children);
	}

	return (HWND)parent;
#endif
}

bool WinInfo::hasNoParent(HWND id)
{
#ifdef Q_OS_WIN
	HWND hParent = GetAncestor(id, GA_PARENT);
	return  ! hParent || hParent == GetDesktopWindow();
#endif
#ifdef Q_OS_LINUX
	unsigned int count;
	Window rootWindow;

	Window wParent = 0;
	Window * wChild = new Window;

	if( XQueryTree(QX11Info::display(), (Window)id, &rootWindow, &wParent, &wChild, &count) != 0)
	{
		XFree(wChild);
	}

	return wParent == 0;
#endif
}

bool WinInfo::isValidWindow( HWND id )
{
#ifdef Q_OS_WIN
	return IsWindow( id );
#endif
#ifdef Q_OS_LINUX
	XWindowAttributes attributes;

	Status ret = XGetWindowAttributes(QX11Info::display(), (Window)id, &attributes);

	if ( ret == 0 )
	{
		return false;
	} else {
		return true;
	}
#endif
	return false;
}

#ifdef Q_OS_LINUX
QString WinInfo::getWindowTextProperty(HWND id, QString property)
{
	XTextProperty tp;
	Atom atom = XInternAtom(QX11Info::display(), property.toUtf8().data(), False);
	Atom utf8 = XInternAtom(QX11Info::display(), "UTF8_STRING", False);
	char ** data = NULL;
	QString result = QString();

	if ( XGetTextProperty( QX11Info::display(), (Window)id, &tp, atom ) && tp.value != NULL )
	{
		if ( tp.encoding == utf8 )
		{
			result = QString::fromUtf8( (const char *)tp.value );
		} else if ( tp.encoding == XA_STRING ) {
			result = QString::fromUtf8( (const char*)tp.value );
		} else {
			result = QString( (const char *)tp.value );
		}
		if( data != NULL )
		{
			XFreeStringList( data );
		}
		XFree( tp.value );
	}

	return result;
}

bool WinInfo::isWindowType(QString windowTypeString, HWND window)
{
	bool ret = false;

	Atom actType;
	int actFormat;
	unsigned long nItems, bytes;
	QList<Atom> windowType;

	Atom netWmType = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", False);

	char * data;
	char * dataPointer;

	int result = XGetWindowProperty(QX11Info::display(), (Window)window, netWmType, 0, 1, False, XA_ATOM, &actType, &actFormat, &nItems, &bytes, (unsigned char **) &data);

	if (result == Success)
	{
		if (data != 0)
		{
			dataPointer = data;
			for ( unsigned int i = 0; i < nItems; i++ )
			{
				if ( *(Atom *) dataPointer == XInternAtom (QX11Info::display(), windowTypeString.toUtf8().data(), False) )
				{
					ret = true;
					i = nItems;
				} else {
					dataPointer++;
				}
			}
			XFree (data);
		}
	}

	return ret;
}

/*QString WinInfo::getWindowType(HWND window)
{

	Atom actType;
	int actFormat;
	unsigned long nItems, bytes;
	Atom windowType;

	Atom netWmName = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", False);

	char * data;

	int result = XGetWindowProperty(QX11Info::display(), (Window)window, netWmName, 0, 1, False, XA_ATOM, &actType, &actFormat, &nItems, &bytes, (unsigned char **) &data);

	if (result == Success)
	{
		if (data != 0)
		{
			windowType = *(Atom *) data;
			XFree (data);
		}
	}

	Display * xdisplay = QX11Info::display();

	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DESKTOP", False))
		return QString("Desktop");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DOCK", False))
		return QString("Dock");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_TOOLBAR", False))
		return QString("Toolbar");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_MENU", False))
		return QString("Menu");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_UTILITY", False))
		return QString("Utility");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_SPLASH", False))
		return QString("Splash");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DIALOG", False))
		return QString("Dialog");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU", False))
		return QString("Dropdown Menu");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_POPUP_MENU", False))
		return QString("Popup Menu");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_TOOLTIP", False))
		return QString("Tooltip");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_NOTIFICATION", False))
		return QString("Notification");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_COMBO", False))
		return QString("Combo");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_DND", False))
		return QString("DND");
	if (windowType == XInternAtom (xdisplay, "_NET_WM_WINDOW_TYPE_NORMAL", False))
		return QString("Normal");

	return QString("Unknown %1").arg((uint)windowType);
}*/
#endif

QSize WinInfo::desktopSize()
{
	return QApplication::primaryScreen()->size();
}

void WinInfo::hideTaskBar()
{
#ifdef Q_OS_WIN
	APPBARDATA barData;
	barData.cbSize = sizeof( barData );
	barData.hWnd = FindWindow( L"System_TrayWnd", NULL);
	barData.lParam = ABS_AUTOHIDE;
	SHAppBarMessage( ABM_SETSTATE, &barData );
#endif
}

void WinInfo::showTaskBar()
{
#ifdef Q_OS_WIN
	APPBARDATA barData;
	barData.cbSize = sizeof( barData );
	barData.hWnd = FindWindow( L"System_TrayWnd", NULL);
	barData.lParam = ABS_ALWAYSONTOP;
	SHAppBarMessage( ABM_SETSTATE, &barData );
#endif
}

QScreen * WinInfo::widgetScreen( QWidget * widget )
{
	QWindow * handle = widget->windowHandle();
	if ( handle != nullptr )
	{
		return handle->screen();
	} else {
		return nullptr;
	}
}
