#ifndef WININFO_H
#define WININFO_H

#include <QObject>
#include <QPixmap>
#include <QImage>
#include <QProcess>
#include <QSize>
#include <QIcon>
#include <QApplication>
#include <QScreen>
#include <QWindow>
#include <QWidget>

#ifdef Q_OS_LINUX
#include <QX11Info>
#endif

#ifdef Q_OS_WIN
#include <WinSock2.h>
#include <Windows.h>
#include <Psapi.h>
#include <LM.h>
#include <WtsApi32.h>
#include <QtWinExtras/QtWin>
#endif

#include "constants.h"

#ifdef Q_OS_WIN
struct handleData
{
	QProcess * process;
	bool systemTray;
	QList<HWND> windows;
	HWND first;
};
BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lparam);
BOOL CALLBACK FindWindowProc(HWND hWnd, LPARAM lparam);
#endif

class WinInfo : public QObject
{
	public:
		static QString getTitle( HWND id );
		static QPixmap getIcon( HWND id );
		static HWND getParent(HWND id);
		static QPixmap toPixmap( HWND id );
		static HWND getWindow(QProcess * process);
		static bool isApplication( HWND id );
		static QSize desktopSize();
		static bool hasNoParent(HWND hWnd);
		static HWND getTopMostWindow();
		static void hideTaskBar();
		static void showTaskBar();
		static bool isValidWindow( HWND id );
		static bool isMinimized( HWND id );

		static QList<HWND> getApplicationWindows();
		static QList<HWND> getSystemTrayWindows();
		static QList<HWND> getWindows(bool systemTrayWindows);

		static QScreen * widgetScreen( QWidget * widget );

#ifdef Q_OS_WIN
		static int isWindowStyle(HWND hWnd, long style);
		static int isWindowExStyle(HWND hWnd, long exStyle);
		static bool isVisible( HWND id );
#endif
#ifdef Q_OS_LINUX
		static QPixmap pixmapFromData(unsigned long *data, int nitems);
		static QPixmap pixmapFromXImage(void * image, void * mask );
		static bool isWindowType(QString windowTypeString, HWND window);
		//static QString getWindowType(HWND window);
		static QString getWindowTextProperty(HWND id, QString property);
		static bool hasState( QString state, HWND id );
#endif
};

#endif
