#!/bin/bash

SRC=src/default
SIZES="16 24 32 48 64 128"

if [ "${1}" == "clean" ]
then
	for i in ${SIZES}
	do
		rm -rf ${i}x${i}
	done
	rm -rf applications groups
	rm -rf ${SRC}/actions/*.png ${SRC}/devices/*.png  ${SRC}/places/*.png
	rm -f ../common/icons
	rm -f ../common/mime
	exit 0
fi

function XCF2PNG()
{
	XCF=$1
	DEST=$2

	PNG=`echo ${XCF} | sed "s/.xcf/.png/g"`

	if [ -n "${DEST}" ]
	then
		NAME=`basename ${PNG}`
		RET=${DEST}/${NAME}
	else
		RET=${PNG}
	fi
	xcf2png -o ${RET} ${XCF}

	echo ${RET}
}

for SIZE in ${SIZES}
do
	mkdir -p ${SIZE}x${SIZE}
done

mkdir -p applications groups

echo "<RCC>" > ../common/resources.qrc
echo -e "    <qresource prefix=\"/\">" >> ../common/resources.qrc

for i in ${SRC}/actions/*.xcf ${SRC}/devices/*.xcf  ${SRC}/places/*.xcf
do
	PNG=`XCF2PNG ${i}`
	NAME=`basename ${PNG}`

	for SIZE in ${SIZES}
	do
		convert -resize ${SIZE}x${SIZE} ${PNG} ${SIZE}x${SIZE}/${NAME}
		echo "        <file>icons/${SIZE}x${SIZE}/${NAME}</file>" >> ../common/resources.qrc
	done
done

for i in src/default/apps/*.xcf
do
	RET=`XCF2PNG ${i} applications`
	NAME=`basename ${RET}`
	echo "        <file>icons/applications/${NAME}</file>" >> ../common/resources.qrc
done

for i in src/default/categories/*.xcf
do
	RET=`XCF2PNG ${i} groups`
	NAME=`basename ${RET}`
	echo "        <file>icons/groups/${NAME}</file>" >> ../common/resources.qrc
done

for i in other/*.png other/*.icns
do
	NAME=`basename ${i}`
	echo "        <file>icons/other/${NAME}</file>" >> ../common/resources.qrc
done

for i in  applications extensions groups types
do
	echo "        <file>mime/${i}</file>" >> ../common/resources.qrc
done

echo "    </qresource>" >> ../common/resources.qrc
echo "</RCC>" >> ../common/resources.qrc

rm -f ../common/icons
rm -f ../common/mime

ln -s ../icons ../common/icons
ln -s ../mime ../common/mime
