#include "basewidget.h"

BaseWidget::BaseWidget(QString widgetName, QString background) : QFrame()
{
	setWindowTitle(widgetName);

	if ( ! background.isNull() )
	{
		pixmap = QPixmap(background);
	}

	setWindowFlags(Qt::FramelessWindowHint);
	setAttribute(Qt::WA_TranslucentBackground);
	setAutoFillBackground(true);
	QVBoxLayout * layout = new QVBoxLayout;
	child = new ChildFrame;
	child->setAutoFillBackground( true );
	layout->setContentsMargins( BLURRADIUS, BLURRADIUS, BLURRADIUS, BLURRADIUS);
	layout->addWidget( child );
	QFrame::setLayout( layout );

	QGraphicsDropShadowEffect * effect = new QGraphicsDropShadowEffect();
	effect->setBlurRadius( BLURRADIUS );
	effect->setXOffset( SHADOWOFFSET );
	effect->setYOffset( SHADOWOFFSET );
	QColor color = SHADOWCOLOR;
	color.setAlpha( SHADOWALPHA );
	effect->setColor( color );
	child->setGraphicsEffect( effect );

	if ( ! pixmap.isNull() )
	{
		child->setPixmap( pixmap );
		setFixedSize(pixmap.width() + 2 * BLURRADIUS, pixmap.height() + 2 * BLURRADIUS);
	}

	/*QString winIdString = QString::number(winId());
	sharedMemory.setKey("Librework " + widgetName);
	if ( ! sharedMemory.create(sizeof(winIdString.toUtf8().data())) )
	{
		qDebug("Cannot create shared memory");
		close();
		return;
	} else {
		sharedMemory.lock();
		char *to = (char*)sharedMemory.data();
		const char *from = winIdString.toUtf8().data();
		memcpy(to, from, sizeof(winIdString.toUtf8().data()));
		sharedMemory.unlock();
	}*/
}

void BaseWidget::setLayout(QLayout * layout)
{
	child->setLayout( layout );
}

void BaseWidget::setLayoutDirection(Qt::LayoutDirection direction)
{
	child->setLayoutDirection( direction );
}

void BaseWidget::setRoundedCorners(int radius)
{
	child->setRadius(radius);
}
